# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os

# try/except added for compatibility with python < 3.8
try:
    from unittest import mock
    from unittest.mock import AsyncMock  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    import mock

from collections.abc import Iterable
import json
import math

from google.api_core import gapic_v1, grpc_helpers, grpc_helpers_async, path_template
from google.api_core import api_core_version, client_options
from google.api_core import exceptions as core_exceptions
import google.auth
from google.auth import credentials as ga_credentials
from google.auth.exceptions import MutualTLSChannelError
from google.geo.type.types import viewport
from google.oauth2 import service_account
from google.protobuf import json_format
from google.type import latlng_pb2  # type: ignore
from google.type import localized_text_pb2  # type: ignore
import grpc
from grpc.experimental import aio
from proto.marshal.rules import wrappers
from proto.marshal.rules.dates import DurationRule, TimestampRule
import pytest
from requests import PreparedRequest, Request, Response
from requests.sessions import Session

from google.maps.places_v1.services.places import (
    PlacesAsyncClient,
    PlacesClient,
    transports,
)
from google.maps.places_v1.types import (
    ev_charging,
    fuel_options,
    geometry,
    photo,
    place,
    places_service,
    review,
)


def client_cert_source_callback():
    return b"cert bytes", b"key bytes"


# If default endpoint is localhost, then default mtls endpoint will be the same.
# This method modifies the default endpoint so the client can produce a different
# mtls endpoint for endpoint testing purposes.
def modify_default_endpoint(client):
    return (
        "foo.googleapis.com"
        if ("localhost" in client.DEFAULT_ENDPOINT)
        else client.DEFAULT_ENDPOINT
    )


# If default endpoint template is localhost, then default mtls endpoint will be the same.
# This method modifies the default endpoint template so the client can produce a different
# mtls endpoint for endpoint testing purposes.
def modify_default_endpoint_template(client):
    return (
        "test.{UNIVERSE_DOMAIN}"
        if ("localhost" in client._DEFAULT_ENDPOINT_TEMPLATE)
        else client._DEFAULT_ENDPOINT_TEMPLATE
    )


def test__get_default_mtls_endpoint():
    api_endpoint = "example.googleapis.com"
    api_mtls_endpoint = "example.mtls.googleapis.com"
    sandbox_endpoint = "example.sandbox.googleapis.com"
    sandbox_mtls_endpoint = "example.mtls.sandbox.googleapis.com"
    non_googleapi = "api.example.com"

    assert PlacesClient._get_default_mtls_endpoint(None) is None
    assert PlacesClient._get_default_mtls_endpoint(api_endpoint) == api_mtls_endpoint
    assert (
        PlacesClient._get_default_mtls_endpoint(api_mtls_endpoint) == api_mtls_endpoint
    )
    assert (
        PlacesClient._get_default_mtls_endpoint(sandbox_endpoint)
        == sandbox_mtls_endpoint
    )
    assert (
        PlacesClient._get_default_mtls_endpoint(sandbox_mtls_endpoint)
        == sandbox_mtls_endpoint
    )
    assert PlacesClient._get_default_mtls_endpoint(non_googleapi) == non_googleapi


def test__read_environment_variables():
    assert PlacesClient._read_environment_variables() == (False, "auto", None)

    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
        assert PlacesClient._read_environment_variables() == (True, "auto", None)

    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "false"}):
        assert PlacesClient._read_environment_variables() == (False, "auto", None)

    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "Unsupported"}
    ):
        with pytest.raises(ValueError) as excinfo:
            PlacesClient._read_environment_variables()
    assert (
        str(excinfo.value)
        == "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
    )

    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
        assert PlacesClient._read_environment_variables() == (False, "never", None)

    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
        assert PlacesClient._read_environment_variables() == (False, "always", None)

    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "auto"}):
        assert PlacesClient._read_environment_variables() == (False, "auto", None)

    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "Unsupported"}):
        with pytest.raises(MutualTLSChannelError) as excinfo:
            PlacesClient._read_environment_variables()
    assert (
        str(excinfo.value)
        == "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
    )

    with mock.patch.dict(os.environ, {"GOOGLE_CLOUD_UNIVERSE_DOMAIN": "foo.com"}):
        assert PlacesClient._read_environment_variables() == (False, "auto", "foo.com")


def test__get_client_cert_source():
    mock_provided_cert_source = mock.Mock()
    mock_default_cert_source = mock.Mock()

    assert PlacesClient._get_client_cert_source(None, False) is None
    assert (
        PlacesClient._get_client_cert_source(mock_provided_cert_source, False) is None
    )
    assert (
        PlacesClient._get_client_cert_source(mock_provided_cert_source, True)
        == mock_provided_cert_source
    )

    with mock.patch(
        "google.auth.transport.mtls.has_default_client_cert_source", return_value=True
    ):
        with mock.patch(
            "google.auth.transport.mtls.default_client_cert_source",
            return_value=mock_default_cert_source,
        ):
            assert (
                PlacesClient._get_client_cert_source(None, True)
                is mock_default_cert_source
            )
            assert (
                PlacesClient._get_client_cert_source(mock_provided_cert_source, "true")
                is mock_provided_cert_source
            )


@mock.patch.object(
    PlacesClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(PlacesClient),
)
@mock.patch.object(
    PlacesAsyncClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(PlacesAsyncClient),
)
def test__get_api_endpoint():
    api_override = "foo.com"
    mock_client_cert_source = mock.Mock()
    default_universe = PlacesClient._DEFAULT_UNIVERSE
    default_endpoint = PlacesClient._DEFAULT_ENDPOINT_TEMPLATE.format(
        UNIVERSE_DOMAIN=default_universe
    )
    mock_universe = "bar.com"
    mock_endpoint = PlacesClient._DEFAULT_ENDPOINT_TEMPLATE.format(
        UNIVERSE_DOMAIN=mock_universe
    )

    assert (
        PlacesClient._get_api_endpoint(
            api_override, mock_client_cert_source, default_universe, "always"
        )
        == api_override
    )
    assert (
        PlacesClient._get_api_endpoint(
            None, mock_client_cert_source, default_universe, "auto"
        )
        == PlacesClient.DEFAULT_MTLS_ENDPOINT
    )
    assert (
        PlacesClient._get_api_endpoint(None, None, default_universe, "auto")
        == default_endpoint
    )
    assert (
        PlacesClient._get_api_endpoint(None, None, default_universe, "always")
        == PlacesClient.DEFAULT_MTLS_ENDPOINT
    )
    assert (
        PlacesClient._get_api_endpoint(
            None, mock_client_cert_source, default_universe, "always"
        )
        == PlacesClient.DEFAULT_MTLS_ENDPOINT
    )
    assert (
        PlacesClient._get_api_endpoint(None, None, mock_universe, "never")
        == mock_endpoint
    )
    assert (
        PlacesClient._get_api_endpoint(None, None, default_universe, "never")
        == default_endpoint
    )

    with pytest.raises(MutualTLSChannelError) as excinfo:
        PlacesClient._get_api_endpoint(
            None, mock_client_cert_source, mock_universe, "auto"
        )
    assert (
        str(excinfo.value)
        == "mTLS is not supported in any universe other than googleapis.com."
    )


def test__get_universe_domain():
    client_universe_domain = "foo.com"
    universe_domain_env = "bar.com"

    assert (
        PlacesClient._get_universe_domain(client_universe_domain, universe_domain_env)
        == client_universe_domain
    )
    assert (
        PlacesClient._get_universe_domain(None, universe_domain_env)
        == universe_domain_env
    )
    assert (
        PlacesClient._get_universe_domain(None, None) == PlacesClient._DEFAULT_UNIVERSE
    )

    with pytest.raises(ValueError) as excinfo:
        PlacesClient._get_universe_domain("", None)
    assert str(excinfo.value) == "Universe Domain cannot be an empty string."


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name",
    [
        (PlacesClient, transports.PlacesGrpcTransport, "grpc"),
        (PlacesClient, transports.PlacesRestTransport, "rest"),
    ],
)
def test__validate_universe_domain(client_class, transport_class, transport_name):
    client = client_class(
        transport=transport_class(credentials=ga_credentials.AnonymousCredentials())
    )
    assert client._validate_universe_domain() == True

    # Test the case when universe is already validated.
    assert client._validate_universe_domain() == True

    if transport_name == "grpc":
        # Test the case where credentials are provided by the
        # `local_channel_credentials`. The default universes in both match.
        channel = grpc.secure_channel(
            "http://localhost/", grpc.local_channel_credentials()
        )
        client = client_class(transport=transport_class(channel=channel))
        assert client._validate_universe_domain() == True

        # Test the case where credentials do not exist: e.g. a transport is provided
        # with no credentials. Validation should still succeed because there is no
        # mismatch with non-existent credentials.
        channel = grpc.secure_channel(
            "http://localhost/", grpc.local_channel_credentials()
        )
        transport = transport_class(channel=channel)
        transport._credentials = None
        client = client_class(transport=transport)
        assert client._validate_universe_domain() == True

    # TODO: This is needed to cater for older versions of google-auth
    # Make this test unconditional once the minimum supported version of
    # google-auth becomes 2.23.0 or higher.
    google_auth_major, google_auth_minor = [
        int(part) for part in google.auth.__version__.split(".")[0:2]
    ]
    if google_auth_major > 2 or (google_auth_major == 2 and google_auth_minor >= 23):
        credentials = ga_credentials.AnonymousCredentials()
        credentials._universe_domain = "foo.com"
        # Test the case when there is a universe mismatch from the credentials.
        client = client_class(transport=transport_class(credentials=credentials))
        with pytest.raises(ValueError) as excinfo:
            client._validate_universe_domain()
        assert (
            str(excinfo.value)
            == "The configured universe domain (googleapis.com) does not match the universe domain found in the credentials (foo.com). If you haven't configured the universe domain explicitly, `googleapis.com` is the default."
        )

        # Test the case when there is a universe mismatch from the client.
        #
        # TODO: Make this test unconditional once the minimum supported version of
        # google-api-core becomes 2.15.0 or higher.
        api_core_major, api_core_minor = [
            int(part) for part in api_core_version.__version__.split(".")[0:2]
        ]
        if api_core_major > 2 or (api_core_major == 2 and api_core_minor >= 15):
            client = client_class(
                client_options={"universe_domain": "bar.com"},
                transport=transport_class(
                    credentials=ga_credentials.AnonymousCredentials(),
                ),
            )
            with pytest.raises(ValueError) as excinfo:
                client._validate_universe_domain()
            assert (
                str(excinfo.value)
                == "The configured universe domain (bar.com) does not match the universe domain found in the credentials (googleapis.com). If you haven't configured the universe domain explicitly, `googleapis.com` is the default."
            )

    # Test that ValueError is raised if universe_domain is provided via client options and credentials is None
    with pytest.raises(ValueError):
        client._compare_universes("foo.bar", None)


@pytest.mark.parametrize(
    "client_class,transport_name",
    [
        (PlacesClient, "grpc"),
        (PlacesAsyncClient, "grpc_asyncio"),
        (PlacesClient, "rest"),
    ],
)
def test_places_client_from_service_account_info(client_class, transport_name):
    creds = ga_credentials.AnonymousCredentials()
    with mock.patch.object(
        service_account.Credentials, "from_service_account_info"
    ) as factory:
        factory.return_value = creds
        info = {"valid": True}
        client = client_class.from_service_account_info(info, transport=transport_name)
        assert client.transport._credentials == creds
        assert isinstance(client, client_class)

        assert client.transport._host == (
            "places.googleapis.com:443"
            if transport_name in ["grpc", "grpc_asyncio"]
            else "https://places.googleapis.com"
        )


@pytest.mark.parametrize(
    "transport_class,transport_name",
    [
        (transports.PlacesGrpcTransport, "grpc"),
        (transports.PlacesGrpcAsyncIOTransport, "grpc_asyncio"),
        (transports.PlacesRestTransport, "rest"),
    ],
)
def test_places_client_service_account_always_use_jwt(transport_class, transport_name):
    with mock.patch.object(
        service_account.Credentials, "with_always_use_jwt_access", create=True
    ) as use_jwt:
        creds = service_account.Credentials(None, None, None)
        transport = transport_class(credentials=creds, always_use_jwt_access=True)
        use_jwt.assert_called_once_with(True)

    with mock.patch.object(
        service_account.Credentials, "with_always_use_jwt_access", create=True
    ) as use_jwt:
        creds = service_account.Credentials(None, None, None)
        transport = transport_class(credentials=creds, always_use_jwt_access=False)
        use_jwt.assert_not_called()


@pytest.mark.parametrize(
    "client_class,transport_name",
    [
        (PlacesClient, "grpc"),
        (PlacesAsyncClient, "grpc_asyncio"),
        (PlacesClient, "rest"),
    ],
)
def test_places_client_from_service_account_file(client_class, transport_name):
    creds = ga_credentials.AnonymousCredentials()
    with mock.patch.object(
        service_account.Credentials, "from_service_account_file"
    ) as factory:
        factory.return_value = creds
        client = client_class.from_service_account_file(
            "dummy/file/path.json", transport=transport_name
        )
        assert client.transport._credentials == creds
        assert isinstance(client, client_class)

        client = client_class.from_service_account_json(
            "dummy/file/path.json", transport=transport_name
        )
        assert client.transport._credentials == creds
        assert isinstance(client, client_class)

        assert client.transport._host == (
            "places.googleapis.com:443"
            if transport_name in ["grpc", "grpc_asyncio"]
            else "https://places.googleapis.com"
        )


def test_places_client_get_transport_class():
    transport = PlacesClient.get_transport_class()
    available_transports = [
        transports.PlacesGrpcTransport,
        transports.PlacesRestTransport,
    ]
    assert transport in available_transports

    transport = PlacesClient.get_transport_class("grpc")
    assert transport == transports.PlacesGrpcTransport


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name",
    [
        (PlacesClient, transports.PlacesGrpcTransport, "grpc"),
        (PlacesAsyncClient, transports.PlacesGrpcAsyncIOTransport, "grpc_asyncio"),
        (PlacesClient, transports.PlacesRestTransport, "rest"),
    ],
)
@mock.patch.object(
    PlacesClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(PlacesClient),
)
@mock.patch.object(
    PlacesAsyncClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(PlacesAsyncClient),
)
def test_places_client_client_options(client_class, transport_class, transport_name):
    # Check that if channel is provided we won't create a new one.
    with mock.patch.object(PlacesClient, "get_transport_class") as gtc:
        transport = transport_class(credentials=ga_credentials.AnonymousCredentials())
        client = client_class(transport=transport)
        gtc.assert_not_called()

    # Check that if channel is provided via str we will create a new one.
    with mock.patch.object(PlacesClient, "get_transport_class") as gtc:
        client = client_class(transport=transport_name)
        gtc.assert_called()

    # Check the case api_endpoint is provided.
    options = client_options.ClientOptions(api_endpoint="squid.clam.whelk")
    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(transport=transport_name, client_options=options)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host="squid.clam.whelk",
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience=None,
        )

    # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT is
    # "never".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
        with mock.patch.object(transport_class, "__init__") as patched:
            patched.return_value = None
            client = client_class(transport=transport_name)
            patched.assert_called_once_with(
                credentials=None,
                credentials_file=None,
                host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                    UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
                ),
                scopes=None,
                client_cert_source_for_mtls=None,
                quota_project_id=None,
                client_info=transports.base.DEFAULT_CLIENT_INFO,
                always_use_jwt_access=True,
                api_audience=None,
            )

    # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT is
    # "always".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
        with mock.patch.object(transport_class, "__init__") as patched:
            patched.return_value = None
            client = client_class(transport=transport_name)
            patched.assert_called_once_with(
                credentials=None,
                credentials_file=None,
                host=client.DEFAULT_MTLS_ENDPOINT,
                scopes=None,
                client_cert_source_for_mtls=None,
                quota_project_id=None,
                client_info=transports.base.DEFAULT_CLIENT_INFO,
                always_use_jwt_access=True,
                api_audience=None,
            )

    # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT has
    # unsupported value.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "Unsupported"}):
        with pytest.raises(MutualTLSChannelError) as excinfo:
            client = client_class(transport=transport_name)
    assert (
        str(excinfo.value)
        == "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
    )

    # Check the case GOOGLE_API_USE_CLIENT_CERTIFICATE has unsupported value.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "Unsupported"}
    ):
        with pytest.raises(ValueError) as excinfo:
            client = client_class(transport=transport_name)
    assert (
        str(excinfo.value)
        == "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
    )

    # Check the case quota_project_id is provided
    options = client_options.ClientOptions(quota_project_id="octopus")
    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options, transport=transport_name)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
            ),
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id="octopus",
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience=None,
        )
    # Check the case api_endpoint is provided
    options = client_options.ClientOptions(
        api_audience="https://language.googleapis.com"
    )
    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options, transport=transport_name)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
            ),
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience="https://language.googleapis.com",
        )


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name,use_client_cert_env",
    [
        (PlacesClient, transports.PlacesGrpcTransport, "grpc", "true"),
        (
            PlacesAsyncClient,
            transports.PlacesGrpcAsyncIOTransport,
            "grpc_asyncio",
            "true",
        ),
        (PlacesClient, transports.PlacesGrpcTransport, "grpc", "false"),
        (
            PlacesAsyncClient,
            transports.PlacesGrpcAsyncIOTransport,
            "grpc_asyncio",
            "false",
        ),
        (PlacesClient, transports.PlacesRestTransport, "rest", "true"),
        (PlacesClient, transports.PlacesRestTransport, "rest", "false"),
    ],
)
@mock.patch.object(
    PlacesClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(PlacesClient),
)
@mock.patch.object(
    PlacesAsyncClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(PlacesAsyncClient),
)
@mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "auto"})
def test_places_client_mtls_env_auto(
    client_class, transport_class, transport_name, use_client_cert_env
):
    # This tests the endpoint autoswitch behavior. Endpoint is autoswitched to the default
    # mtls endpoint, if GOOGLE_API_USE_CLIENT_CERTIFICATE is "true" and client cert exists.

    # Check the case client_cert_source is provided. Whether client cert is used depends on
    # GOOGLE_API_USE_CLIENT_CERTIFICATE value.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": use_client_cert_env}
    ):
        options = client_options.ClientOptions(
            client_cert_source=client_cert_source_callback
        )
        with mock.patch.object(transport_class, "__init__") as patched:
            patched.return_value = None
            client = client_class(client_options=options, transport=transport_name)

            if use_client_cert_env == "false":
                expected_client_cert_source = None
                expected_host = client._DEFAULT_ENDPOINT_TEMPLATE.format(
                    UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
                )
            else:
                expected_client_cert_source = client_cert_source_callback
                expected_host = client.DEFAULT_MTLS_ENDPOINT

            patched.assert_called_once_with(
                credentials=None,
                credentials_file=None,
                host=expected_host,
                scopes=None,
                client_cert_source_for_mtls=expected_client_cert_source,
                quota_project_id=None,
                client_info=transports.base.DEFAULT_CLIENT_INFO,
                always_use_jwt_access=True,
                api_audience=None,
            )

    # Check the case ADC client cert is provided. Whether client cert is used depends on
    # GOOGLE_API_USE_CLIENT_CERTIFICATE value.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": use_client_cert_env}
    ):
        with mock.patch.object(transport_class, "__init__") as patched:
            with mock.patch(
                "google.auth.transport.mtls.has_default_client_cert_source",
                return_value=True,
            ):
                with mock.patch(
                    "google.auth.transport.mtls.default_client_cert_source",
                    return_value=client_cert_source_callback,
                ):
                    if use_client_cert_env == "false":
                        expected_host = client._DEFAULT_ENDPOINT_TEMPLATE.format(
                            UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
                        )
                        expected_client_cert_source = None
                    else:
                        expected_host = client.DEFAULT_MTLS_ENDPOINT
                        expected_client_cert_source = client_cert_source_callback

                    patched.return_value = None
                    client = client_class(transport=transport_name)
                    patched.assert_called_once_with(
                        credentials=None,
                        credentials_file=None,
                        host=expected_host,
                        scopes=None,
                        client_cert_source_for_mtls=expected_client_cert_source,
                        quota_project_id=None,
                        client_info=transports.base.DEFAULT_CLIENT_INFO,
                        always_use_jwt_access=True,
                        api_audience=None,
                    )

    # Check the case client_cert_source and ADC client cert are not provided.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": use_client_cert_env}
    ):
        with mock.patch.object(transport_class, "__init__") as patched:
            with mock.patch(
                "google.auth.transport.mtls.has_default_client_cert_source",
                return_value=False,
            ):
                patched.return_value = None
                client = client_class(transport=transport_name)
                patched.assert_called_once_with(
                    credentials=None,
                    credentials_file=None,
                    host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                        UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
                    ),
                    scopes=None,
                    client_cert_source_for_mtls=None,
                    quota_project_id=None,
                    client_info=transports.base.DEFAULT_CLIENT_INFO,
                    always_use_jwt_access=True,
                    api_audience=None,
                )


@pytest.mark.parametrize("client_class", [PlacesClient, PlacesAsyncClient])
@mock.patch.object(
    PlacesClient, "DEFAULT_ENDPOINT", modify_default_endpoint(PlacesClient)
)
@mock.patch.object(
    PlacesAsyncClient, "DEFAULT_ENDPOINT", modify_default_endpoint(PlacesAsyncClient)
)
def test_places_client_get_mtls_endpoint_and_cert_source(client_class):
    mock_client_cert_source = mock.Mock()

    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "true".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
        mock_api_endpoint = "foo"
        options = client_options.ClientOptions(
            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
        )
        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
            options
        )
        assert api_endpoint == mock_api_endpoint
        assert cert_source == mock_client_cert_source

    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "false".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "false"}):
        mock_client_cert_source = mock.Mock()
        mock_api_endpoint = "foo"
        options = client_options.ClientOptions(
            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
        )
        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
            options
        )
        assert api_endpoint == mock_api_endpoint
        assert cert_source is None

    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "never".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
        assert api_endpoint == client_class.DEFAULT_ENDPOINT
        assert cert_source is None

    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "always".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
        assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
        assert cert_source is None

    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert doesn't exist.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
        with mock.patch(
            "google.auth.transport.mtls.has_default_client_cert_source",
            return_value=False,
        ):
            api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
            assert api_endpoint == client_class.DEFAULT_ENDPOINT
            assert cert_source is None

    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert exists.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
        with mock.patch(
            "google.auth.transport.mtls.has_default_client_cert_source",
            return_value=True,
        ):
            with mock.patch(
                "google.auth.transport.mtls.default_client_cert_source",
                return_value=mock_client_cert_source,
            ):
                (
                    api_endpoint,
                    cert_source,
                ) = client_class.get_mtls_endpoint_and_cert_source()
                assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
                assert cert_source == mock_client_cert_source

    # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT has
    # unsupported value.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "Unsupported"}):
        with pytest.raises(MutualTLSChannelError) as excinfo:
            client_class.get_mtls_endpoint_and_cert_source()

        assert (
            str(excinfo.value)
            == "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
        )

    # Check the case GOOGLE_API_USE_CLIENT_CERTIFICATE has unsupported value.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "Unsupported"}
    ):
        with pytest.raises(ValueError) as excinfo:
            client_class.get_mtls_endpoint_and_cert_source()

        assert (
            str(excinfo.value)
            == "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
        )


@pytest.mark.parametrize("client_class", [PlacesClient, PlacesAsyncClient])
@mock.patch.object(
    PlacesClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(PlacesClient),
)
@mock.patch.object(
    PlacesAsyncClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(PlacesAsyncClient),
)
def test_places_client_client_api_endpoint(client_class):
    mock_client_cert_source = client_cert_source_callback
    api_override = "foo.com"
    default_universe = PlacesClient._DEFAULT_UNIVERSE
    default_endpoint = PlacesClient._DEFAULT_ENDPOINT_TEMPLATE.format(
        UNIVERSE_DOMAIN=default_universe
    )
    mock_universe = "bar.com"
    mock_endpoint = PlacesClient._DEFAULT_ENDPOINT_TEMPLATE.format(
        UNIVERSE_DOMAIN=mock_universe
    )

    # If ClientOptions.api_endpoint is set and GOOGLE_API_USE_CLIENT_CERTIFICATE="true",
    # use ClientOptions.api_endpoint as the api endpoint regardless.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
        with mock.patch(
            "google.auth.transport.requests.AuthorizedSession.configure_mtls_channel"
        ):
            options = client_options.ClientOptions(
                client_cert_source=mock_client_cert_source, api_endpoint=api_override
            )
            client = client_class(
                client_options=options,
                credentials=ga_credentials.AnonymousCredentials(),
            )
            assert client.api_endpoint == api_override

    # If ClientOptions.api_endpoint is not set and GOOGLE_API_USE_MTLS_ENDPOINT="never",
    # use the _DEFAULT_ENDPOINT_TEMPLATE populated with GDU as the api endpoint.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
        client = client_class(credentials=ga_credentials.AnonymousCredentials())
        assert client.api_endpoint == default_endpoint

    # If ClientOptions.api_endpoint is not set and GOOGLE_API_USE_MTLS_ENDPOINT="always",
    # use the DEFAULT_MTLS_ENDPOINT as the api endpoint.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
        client = client_class(credentials=ga_credentials.AnonymousCredentials())
        assert client.api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT

    # If ClientOptions.api_endpoint is not set, GOOGLE_API_USE_MTLS_ENDPOINT="auto" (default),
    # GOOGLE_API_USE_CLIENT_CERTIFICATE="false" (default), default cert source doesn't exist,
    # and ClientOptions.universe_domain="bar.com",
    # use the _DEFAULT_ENDPOINT_TEMPLATE populated with universe domain as the api endpoint.
    options = client_options.ClientOptions()
    universe_exists = hasattr(options, "universe_domain")
    if universe_exists:
        options = client_options.ClientOptions(universe_domain=mock_universe)
        client = client_class(
            client_options=options, credentials=ga_credentials.AnonymousCredentials()
        )
    else:
        client = client_class(
            client_options=options, credentials=ga_credentials.AnonymousCredentials()
        )
    assert client.api_endpoint == (
        mock_endpoint if universe_exists else default_endpoint
    )
    assert client.universe_domain == (
        mock_universe if universe_exists else default_universe
    )

    # If ClientOptions does not have a universe domain attribute and GOOGLE_API_USE_MTLS_ENDPOINT="never",
    # use the _DEFAULT_ENDPOINT_TEMPLATE populated with GDU as the api endpoint.
    options = client_options.ClientOptions()
    if hasattr(options, "universe_domain"):
        delattr(options, "universe_domain")
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
        client = client_class(
            client_options=options, credentials=ga_credentials.AnonymousCredentials()
        )
        assert client.api_endpoint == default_endpoint


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name",
    [
        (PlacesClient, transports.PlacesGrpcTransport, "grpc"),
        (PlacesAsyncClient, transports.PlacesGrpcAsyncIOTransport, "grpc_asyncio"),
        (PlacesClient, transports.PlacesRestTransport, "rest"),
    ],
)
def test_places_client_client_options_scopes(
    client_class, transport_class, transport_name
):
    # Check the case scopes are provided.
    options = client_options.ClientOptions(
        scopes=["1", "2"],
    )
    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options, transport=transport_name)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
            ),
            scopes=["1", "2"],
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience=None,
        )


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name,grpc_helpers",
    [
        (PlacesClient, transports.PlacesGrpcTransport, "grpc", grpc_helpers),
        (
            PlacesAsyncClient,
            transports.PlacesGrpcAsyncIOTransport,
            "grpc_asyncio",
            grpc_helpers_async,
        ),
        (PlacesClient, transports.PlacesRestTransport, "rest", None),
    ],
)
def test_places_client_client_options_credentials_file(
    client_class, transport_class, transport_name, grpc_helpers
):
    # Check the case credentials file is provided.
    options = client_options.ClientOptions(credentials_file="credentials.json")

    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options, transport=transport_name)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file="credentials.json",
            host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
            ),
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience=None,
        )


def test_places_client_client_options_from_dict():
    with mock.patch(
        "google.maps.places_v1.services.places.transports.PlacesGrpcTransport.__init__"
    ) as grpc_transport:
        grpc_transport.return_value = None
        client = PlacesClient(client_options={"api_endpoint": "squid.clam.whelk"})
        grpc_transport.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host="squid.clam.whelk",
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience=None,
        )


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name,grpc_helpers",
    [
        (PlacesClient, transports.PlacesGrpcTransport, "grpc", grpc_helpers),
        (
            PlacesAsyncClient,
            transports.PlacesGrpcAsyncIOTransport,
            "grpc_asyncio",
            grpc_helpers_async,
        ),
    ],
)
def test_places_client_create_channel_credentials_file(
    client_class, transport_class, transport_name, grpc_helpers
):
    # Check the case credentials file is provided.
    options = client_options.ClientOptions(credentials_file="credentials.json")

    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options, transport=transport_name)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file="credentials.json",
            host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
            ),
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience=None,
        )

    # test that the credentials from file are saved and used as the credentials.
    with mock.patch.object(
        google.auth, "load_credentials_from_file", autospec=True
    ) as load_creds, mock.patch.object(
        google.auth, "default", autospec=True
    ) as adc, mock.patch.object(
        grpc_helpers, "create_channel"
    ) as create_channel:
        creds = ga_credentials.AnonymousCredentials()
        file_creds = ga_credentials.AnonymousCredentials()
        load_creds.return_value = (file_creds, None)
        adc.return_value = (creds, None)
        client = client_class(client_options=options, transport=transport_name)
        create_channel.assert_called_with(
            "places.googleapis.com:443",
            credentials=file_creds,
            credentials_file=None,
            quota_project_id=None,
            default_scopes=(),
            scopes=None,
            default_host="places.googleapis.com",
            ssl_credentials=None,
            options=[
                ("grpc.max_send_message_length", -1),
                ("grpc.max_receive_message_length", -1),
            ],
        )


@pytest.mark.parametrize(
    "request_type",
    [
        places_service.SearchNearbyRequest,
        dict,
    ],
)
def test_search_nearby(request_type, transport: str = "grpc"):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.search_nearby), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = places_service.SearchNearbyResponse()
        response = client.search_nearby(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        request = places_service.SearchNearbyRequest()
        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, places_service.SearchNearbyResponse)


def test_search_nearby_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.search_nearby), "__call__") as call:
        client.search_nearby()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.SearchNearbyRequest()


def test_search_nearby_non_empty_request_with_auto_populated_field():
    # This test is a coverage failsafe to make sure that UUID4 fields are
    # automatically populated, according to AIP-4235, with non-empty requests.
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc",
    )

    # Populate all string fields in the request which are not UUID4
    # since we want to check that UUID4 are populated automatically
    # if they meet the requirements of AIP 4235.
    request = places_service.SearchNearbyRequest(
        language_code="language_code_value",
        region_code="region_code_value",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.search_nearby), "__call__") as call:
        client.search_nearby(request=request)
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.SearchNearbyRequest(
            language_code="language_code_value",
            region_code="region_code_value",
        )


@pytest.mark.asyncio
async def test_search_nearby_empty_call_async():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc_asyncio",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.search_nearby), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            places_service.SearchNearbyResponse()
        )
        response = await client.search_nearby()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.SearchNearbyRequest()


@pytest.mark.asyncio
async def test_search_nearby_async(
    transport: str = "grpc_asyncio", request_type=places_service.SearchNearbyRequest
):
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.search_nearby), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            places_service.SearchNearbyResponse()
        )
        response = await client.search_nearby(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        request = places_service.SearchNearbyRequest()
        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, places_service.SearchNearbyResponse)


@pytest.mark.asyncio
async def test_search_nearby_async_from_dict():
    await test_search_nearby_async(request_type=dict)


@pytest.mark.parametrize(
    "request_type",
    [
        places_service.SearchTextRequest,
        dict,
    ],
)
def test_search_text(request_type, transport: str = "grpc"):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.search_text), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = places_service.SearchTextResponse()
        response = client.search_text(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        request = places_service.SearchTextRequest()
        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, places_service.SearchTextResponse)


def test_search_text_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.search_text), "__call__") as call:
        client.search_text()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.SearchTextRequest()


def test_search_text_non_empty_request_with_auto_populated_field():
    # This test is a coverage failsafe to make sure that UUID4 fields are
    # automatically populated, according to AIP-4235, with non-empty requests.
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc",
    )

    # Populate all string fields in the request which are not UUID4
    # since we want to check that UUID4 are populated automatically
    # if they meet the requirements of AIP 4235.
    request = places_service.SearchTextRequest(
        text_query="text_query_value",
        language_code="language_code_value",
        region_code="region_code_value",
        included_type="included_type_value",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.search_text), "__call__") as call:
        client.search_text(request=request)
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.SearchTextRequest(
            text_query="text_query_value",
            language_code="language_code_value",
            region_code="region_code_value",
            included_type="included_type_value",
        )


@pytest.mark.asyncio
async def test_search_text_empty_call_async():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc_asyncio",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.search_text), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            places_service.SearchTextResponse()
        )
        response = await client.search_text()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.SearchTextRequest()


@pytest.mark.asyncio
async def test_search_text_async(
    transport: str = "grpc_asyncio", request_type=places_service.SearchTextRequest
):
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.search_text), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            places_service.SearchTextResponse()
        )
        response = await client.search_text(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        request = places_service.SearchTextRequest()
        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, places_service.SearchTextResponse)


@pytest.mark.asyncio
async def test_search_text_async_from_dict():
    await test_search_text_async(request_type=dict)


@pytest.mark.parametrize(
    "request_type",
    [
        places_service.GetPhotoMediaRequest,
        dict,
    ],
)
def test_get_photo_media(request_type, transport: str = "grpc"):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_photo_media), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = places_service.PhotoMedia(
            name="name_value",
            photo_uri="photo_uri_value",
        )
        response = client.get_photo_media(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        request = places_service.GetPhotoMediaRequest()
        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, places_service.PhotoMedia)
    assert response.name == "name_value"
    assert response.photo_uri == "photo_uri_value"


def test_get_photo_media_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_photo_media), "__call__") as call:
        client.get_photo_media()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.GetPhotoMediaRequest()


def test_get_photo_media_non_empty_request_with_auto_populated_field():
    # This test is a coverage failsafe to make sure that UUID4 fields are
    # automatically populated, according to AIP-4235, with non-empty requests.
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc",
    )

    # Populate all string fields in the request which are not UUID4
    # since we want to check that UUID4 are populated automatically
    # if they meet the requirements of AIP 4235.
    request = places_service.GetPhotoMediaRequest(
        name="name_value",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_photo_media), "__call__") as call:
        client.get_photo_media(request=request)
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.GetPhotoMediaRequest(
            name="name_value",
        )


@pytest.mark.asyncio
async def test_get_photo_media_empty_call_async():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc_asyncio",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_photo_media), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            places_service.PhotoMedia(
                name="name_value",
                photo_uri="photo_uri_value",
            )
        )
        response = await client.get_photo_media()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.GetPhotoMediaRequest()


@pytest.mark.asyncio
async def test_get_photo_media_async(
    transport: str = "grpc_asyncio", request_type=places_service.GetPhotoMediaRequest
):
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_photo_media), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            places_service.PhotoMedia(
                name="name_value",
                photo_uri="photo_uri_value",
            )
        )
        response = await client.get_photo_media(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        request = places_service.GetPhotoMediaRequest()
        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, places_service.PhotoMedia)
    assert response.name == "name_value"
    assert response.photo_uri == "photo_uri_value"


@pytest.mark.asyncio
async def test_get_photo_media_async_from_dict():
    await test_get_photo_media_async(request_type=dict)


def test_get_photo_media_field_headers():
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = places_service.GetPhotoMediaRequest()

    request.name = "name_value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_photo_media), "__call__") as call:
        call.return_value = places_service.PhotoMedia()
        client.get_photo_media(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert (
        "x-goog-request-params",
        "name=name_value",
    ) in kw["metadata"]


@pytest.mark.asyncio
async def test_get_photo_media_field_headers_async():
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = places_service.GetPhotoMediaRequest()

    request.name = "name_value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_photo_media), "__call__") as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            places_service.PhotoMedia()
        )
        await client.get_photo_media(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert (
        "x-goog-request-params",
        "name=name_value",
    ) in kw["metadata"]


def test_get_photo_media_flattened():
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_photo_media), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = places_service.PhotoMedia()
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.get_photo_media(
            name="name_value",
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        arg = args[0].name
        mock_val = "name_value"
        assert arg == mock_val


def test_get_photo_media_flattened_error():
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.get_photo_media(
            places_service.GetPhotoMediaRequest(),
            name="name_value",
        )


@pytest.mark.asyncio
async def test_get_photo_media_flattened_async():
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_photo_media), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = places_service.PhotoMedia()

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            places_service.PhotoMedia()
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.get_photo_media(
            name="name_value",
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        arg = args[0].name
        mock_val = "name_value"
        assert arg == mock_val


@pytest.mark.asyncio
async def test_get_photo_media_flattened_error_async():
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.get_photo_media(
            places_service.GetPhotoMediaRequest(),
            name="name_value",
        )


@pytest.mark.parametrize(
    "request_type",
    [
        places_service.GetPlaceRequest,
        dict,
    ],
)
def test_get_place(request_type, transport: str = "grpc"):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_place), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = place.Place(
            name="name_value",
            id="id_value",
            types=["types_value"],
            primary_type="primary_type_value",
            national_phone_number="national_phone_number_value",
            international_phone_number="international_phone_number_value",
            formatted_address="formatted_address_value",
            short_formatted_address="short_formatted_address_value",
            rating=0.645,
            google_maps_uri="google_maps_uri_value",
            website_uri="website_uri_value",
            utc_offset_minutes=1942,
            adr_format_address="adr_format_address_value",
            business_status=place.Place.BusinessStatus.OPERATIONAL,
            price_level=place.PriceLevel.PRICE_LEVEL_FREE,
            user_rating_count=1835,
            icon_mask_base_uri="icon_mask_base_uri_value",
            icon_background_color="icon_background_color_value",
            takeout=True,
            delivery=True,
            dine_in=True,
            curbside_pickup=True,
            reservable=True,
            serves_breakfast=True,
            serves_lunch=True,
            serves_dinner=True,
            serves_beer=True,
            serves_wine=True,
            serves_brunch=True,
            serves_vegetarian_food=True,
            outdoor_seating=True,
            live_music=True,
            menu_for_children=True,
            serves_cocktails=True,
            serves_dessert=True,
            serves_coffee=True,
            good_for_children=True,
            allows_dogs=True,
            restroom=True,
            good_for_groups=True,
            good_for_watching_sports=True,
        )
        response = client.get_place(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        request = places_service.GetPlaceRequest()
        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, place.Place)
    assert response.name == "name_value"
    assert response.id == "id_value"
    assert response.types == ["types_value"]
    assert response.primary_type == "primary_type_value"
    assert response.national_phone_number == "national_phone_number_value"
    assert response.international_phone_number == "international_phone_number_value"
    assert response.formatted_address == "formatted_address_value"
    assert response.short_formatted_address == "short_formatted_address_value"
    assert math.isclose(response.rating, 0.645, rel_tol=1e-6)
    assert response.google_maps_uri == "google_maps_uri_value"
    assert response.website_uri == "website_uri_value"
    assert response.utc_offset_minutes == 1942
    assert response.adr_format_address == "adr_format_address_value"
    assert response.business_status == place.Place.BusinessStatus.OPERATIONAL
    assert response.price_level == place.PriceLevel.PRICE_LEVEL_FREE
    assert response.user_rating_count == 1835
    assert response.icon_mask_base_uri == "icon_mask_base_uri_value"
    assert response.icon_background_color == "icon_background_color_value"
    assert response.takeout is True
    assert response.delivery is True
    assert response.dine_in is True
    assert response.curbside_pickup is True
    assert response.reservable is True
    assert response.serves_breakfast is True
    assert response.serves_lunch is True
    assert response.serves_dinner is True
    assert response.serves_beer is True
    assert response.serves_wine is True
    assert response.serves_brunch is True
    assert response.serves_vegetarian_food is True
    assert response.outdoor_seating is True
    assert response.live_music is True
    assert response.menu_for_children is True
    assert response.serves_cocktails is True
    assert response.serves_dessert is True
    assert response.serves_coffee is True
    assert response.good_for_children is True
    assert response.allows_dogs is True
    assert response.restroom is True
    assert response.good_for_groups is True
    assert response.good_for_watching_sports is True


def test_get_place_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_place), "__call__") as call:
        client.get_place()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.GetPlaceRequest()


def test_get_place_non_empty_request_with_auto_populated_field():
    # This test is a coverage failsafe to make sure that UUID4 fields are
    # automatically populated, according to AIP-4235, with non-empty requests.
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc",
    )

    # Populate all string fields in the request which are not UUID4
    # since we want to check that UUID4 are populated automatically
    # if they meet the requirements of AIP 4235.
    request = places_service.GetPlaceRequest(
        name="name_value",
        language_code="language_code_value",
        region_code="region_code_value",
        session_token="session_token_value",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_place), "__call__") as call:
        client.get_place(request=request)
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.GetPlaceRequest(
            name="name_value",
            language_code="language_code_value",
            region_code="region_code_value",
            session_token="session_token_value",
        )


@pytest.mark.asyncio
async def test_get_place_empty_call_async():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc_asyncio",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_place), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            place.Place(
                name="name_value",
                id="id_value",
                types=["types_value"],
                primary_type="primary_type_value",
                national_phone_number="national_phone_number_value",
                international_phone_number="international_phone_number_value",
                formatted_address="formatted_address_value",
                short_formatted_address="short_formatted_address_value",
                rating=0.645,
                google_maps_uri="google_maps_uri_value",
                website_uri="website_uri_value",
                utc_offset_minutes=1942,
                adr_format_address="adr_format_address_value",
                business_status=place.Place.BusinessStatus.OPERATIONAL,
                price_level=place.PriceLevel.PRICE_LEVEL_FREE,
                user_rating_count=1835,
                icon_mask_base_uri="icon_mask_base_uri_value",
                icon_background_color="icon_background_color_value",
                takeout=True,
                delivery=True,
                dine_in=True,
                curbside_pickup=True,
                reservable=True,
                serves_breakfast=True,
                serves_lunch=True,
                serves_dinner=True,
                serves_beer=True,
                serves_wine=True,
                serves_brunch=True,
                serves_vegetarian_food=True,
                outdoor_seating=True,
                live_music=True,
                menu_for_children=True,
                serves_cocktails=True,
                serves_dessert=True,
                serves_coffee=True,
                good_for_children=True,
                allows_dogs=True,
                restroom=True,
                good_for_groups=True,
                good_for_watching_sports=True,
            )
        )
        response = await client.get_place()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.GetPlaceRequest()


@pytest.mark.asyncio
async def test_get_place_async(
    transport: str = "grpc_asyncio", request_type=places_service.GetPlaceRequest
):
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_place), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            place.Place(
                name="name_value",
                id="id_value",
                types=["types_value"],
                primary_type="primary_type_value",
                national_phone_number="national_phone_number_value",
                international_phone_number="international_phone_number_value",
                formatted_address="formatted_address_value",
                short_formatted_address="short_formatted_address_value",
                rating=0.645,
                google_maps_uri="google_maps_uri_value",
                website_uri="website_uri_value",
                utc_offset_minutes=1942,
                adr_format_address="adr_format_address_value",
                business_status=place.Place.BusinessStatus.OPERATIONAL,
                price_level=place.PriceLevel.PRICE_LEVEL_FREE,
                user_rating_count=1835,
                icon_mask_base_uri="icon_mask_base_uri_value",
                icon_background_color="icon_background_color_value",
                takeout=True,
                delivery=True,
                dine_in=True,
                curbside_pickup=True,
                reservable=True,
                serves_breakfast=True,
                serves_lunch=True,
                serves_dinner=True,
                serves_beer=True,
                serves_wine=True,
                serves_brunch=True,
                serves_vegetarian_food=True,
                outdoor_seating=True,
                live_music=True,
                menu_for_children=True,
                serves_cocktails=True,
                serves_dessert=True,
                serves_coffee=True,
                good_for_children=True,
                allows_dogs=True,
                restroom=True,
                good_for_groups=True,
                good_for_watching_sports=True,
            )
        )
        response = await client.get_place(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        request = places_service.GetPlaceRequest()
        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, place.Place)
    assert response.name == "name_value"
    assert response.id == "id_value"
    assert response.types == ["types_value"]
    assert response.primary_type == "primary_type_value"
    assert response.national_phone_number == "national_phone_number_value"
    assert response.international_phone_number == "international_phone_number_value"
    assert response.formatted_address == "formatted_address_value"
    assert response.short_formatted_address == "short_formatted_address_value"
    assert math.isclose(response.rating, 0.645, rel_tol=1e-6)
    assert response.google_maps_uri == "google_maps_uri_value"
    assert response.website_uri == "website_uri_value"
    assert response.utc_offset_minutes == 1942
    assert response.adr_format_address == "adr_format_address_value"
    assert response.business_status == place.Place.BusinessStatus.OPERATIONAL
    assert response.price_level == place.PriceLevel.PRICE_LEVEL_FREE
    assert response.user_rating_count == 1835
    assert response.icon_mask_base_uri == "icon_mask_base_uri_value"
    assert response.icon_background_color == "icon_background_color_value"
    assert response.takeout is True
    assert response.delivery is True
    assert response.dine_in is True
    assert response.curbside_pickup is True
    assert response.reservable is True
    assert response.serves_breakfast is True
    assert response.serves_lunch is True
    assert response.serves_dinner is True
    assert response.serves_beer is True
    assert response.serves_wine is True
    assert response.serves_brunch is True
    assert response.serves_vegetarian_food is True
    assert response.outdoor_seating is True
    assert response.live_music is True
    assert response.menu_for_children is True
    assert response.serves_cocktails is True
    assert response.serves_dessert is True
    assert response.serves_coffee is True
    assert response.good_for_children is True
    assert response.allows_dogs is True
    assert response.restroom is True
    assert response.good_for_groups is True
    assert response.good_for_watching_sports is True


@pytest.mark.asyncio
async def test_get_place_async_from_dict():
    await test_get_place_async(request_type=dict)


def test_get_place_field_headers():
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = places_service.GetPlaceRequest()

    request.name = "name_value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_place), "__call__") as call:
        call.return_value = place.Place()
        client.get_place(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert (
        "x-goog-request-params",
        "name=name_value",
    ) in kw["metadata"]


@pytest.mark.asyncio
async def test_get_place_field_headers_async():
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = places_service.GetPlaceRequest()

    request.name = "name_value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_place), "__call__") as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(place.Place())
        await client.get_place(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert (
        "x-goog-request-params",
        "name=name_value",
    ) in kw["metadata"]


def test_get_place_flattened():
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_place), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = place.Place()
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.get_place(
            name="name_value",
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        arg = args[0].name
        mock_val = "name_value"
        assert arg == mock_val


def test_get_place_flattened_error():
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.get_place(
            places_service.GetPlaceRequest(),
            name="name_value",
        )


@pytest.mark.asyncio
async def test_get_place_flattened_async():
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_place), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = place.Place()

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(place.Place())
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.get_place(
            name="name_value",
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        arg = args[0].name
        mock_val = "name_value"
        assert arg == mock_val


@pytest.mark.asyncio
async def test_get_place_flattened_error_async():
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.get_place(
            places_service.GetPlaceRequest(),
            name="name_value",
        )


@pytest.mark.parametrize(
    "request_type",
    [
        places_service.AutocompletePlacesRequest,
        dict,
    ],
)
def test_autocomplete_places(request_type, transport: str = "grpc"):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.autocomplete_places), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = places_service.AutocompletePlacesResponse()
        response = client.autocomplete_places(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        request = places_service.AutocompletePlacesRequest()
        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, places_service.AutocompletePlacesResponse)


def test_autocomplete_places_empty_call():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.autocomplete_places), "__call__"
    ) as call:
        client.autocomplete_places()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.AutocompletePlacesRequest()


def test_autocomplete_places_non_empty_request_with_auto_populated_field():
    # This test is a coverage failsafe to make sure that UUID4 fields are
    # automatically populated, according to AIP-4235, with non-empty requests.
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc",
    )

    # Populate all string fields in the request which are not UUID4
    # since we want to check that UUID4 are populated automatically
    # if they meet the requirements of AIP 4235.
    request = places_service.AutocompletePlacesRequest(
        input="input_value",
        language_code="language_code_value",
        region_code="region_code_value",
        session_token="session_token_value",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.autocomplete_places), "__call__"
    ) as call:
        client.autocomplete_places(request=request)
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.AutocompletePlacesRequest(
            input="input_value",
            language_code="language_code_value",
            region_code="region_code_value",
            session_token="session_token_value",
        )


@pytest.mark.asyncio
async def test_autocomplete_places_empty_call_async():
    # This test is a coverage failsafe to make sure that totally empty calls,
    # i.e. request == None and no flattened fields passed, work.
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc_asyncio",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.autocomplete_places), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            places_service.AutocompletePlacesResponse()
        )
        response = await client.autocomplete_places()
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == places_service.AutocompletePlacesRequest()


@pytest.mark.asyncio
async def test_autocomplete_places_async(
    transport: str = "grpc_asyncio",
    request_type=places_service.AutocompletePlacesRequest,
):
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(
        type(client.transport.autocomplete_places), "__call__"
    ) as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            places_service.AutocompletePlacesResponse()
        )
        response = await client.autocomplete_places(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        request = places_service.AutocompletePlacesRequest()
        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, places_service.AutocompletePlacesResponse)


@pytest.mark.asyncio
async def test_autocomplete_places_async_from_dict():
    await test_autocomplete_places_async(request_type=dict)


@pytest.mark.parametrize(
    "request_type",
    [
        places_service.SearchNearbyRequest,
        dict,
    ],
)
def test_search_nearby_rest(request_type):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )

    # send a request that will satisfy transcoding
    request_init = {}
    request = request_type(**request_init)

    # Mock the http request call within the method and fake a response.
    with mock.patch.object(type(client.transport._session), "request") as req:
        # Designate an appropriate value for the returned response.
        return_value = places_service.SearchNearbyResponse()

        # Wrap the value into a proper Response obj
        response_value = Response()
        response_value.status_code = 200
        # Convert return value to protobuf type
        return_value = places_service.SearchNearbyResponse.pb(return_value)
        json_return_value = json_format.MessageToJson(return_value)

        response_value._content = json_return_value.encode("UTF-8")
        req.return_value = response_value
        response = client.search_nearby(request)

    # Establish that the response is the type that we expect.
    assert isinstance(response, places_service.SearchNearbyResponse)


def test_search_nearby_rest_required_fields(
    request_type=places_service.SearchNearbyRequest,
):
    transport_class = transports.PlacesRestTransport

    request_init = {}
    request = request_type(**request_init)
    pb_request = request_type.pb(request)
    jsonified_request = json.loads(
        json_format.MessageToJson(pb_request, use_integers_for_enums=False)
    )

    # verify fields with default values are dropped

    unset_fields = transport_class(
        credentials=ga_credentials.AnonymousCredentials()
    ).search_nearby._get_unset_required_fields(jsonified_request)
    jsonified_request.update(unset_fields)

    # verify required fields with default values are now present

    unset_fields = transport_class(
        credentials=ga_credentials.AnonymousCredentials()
    ).search_nearby._get_unset_required_fields(jsonified_request)
    jsonified_request.update(unset_fields)

    # verify required fields with non-default values are left alone

    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )
    request = request_type(**request_init)

    # Designate an appropriate value for the returned response.
    return_value = places_service.SearchNearbyResponse()
    # Mock the http request call within the method and fake a response.
    with mock.patch.object(Session, "request") as req:
        # We need to mock transcode() because providing default values
        # for required fields will fail the real version if the http_options
        # expect actual values for those fields.
        with mock.patch.object(path_template, "transcode") as transcode:
            # A uri without fields and an empty body will force all the
            # request fields to show up in the query_params.
            pb_request = request_type.pb(request)
            transcode_result = {
                "uri": "v1/sample_method",
                "method": "post",
                "query_params": pb_request,
            }
            transcode_result["body"] = pb_request
            transcode.return_value = transcode_result

            response_value = Response()
            response_value.status_code = 200

            # Convert return value to protobuf type
            return_value = places_service.SearchNearbyResponse.pb(return_value)
            json_return_value = json_format.MessageToJson(return_value)

            response_value._content = json_return_value.encode("UTF-8")
            req.return_value = response_value

            response = client.search_nearby(request)

            expected_params = [("$alt", "json;enum-encoding=int")]
            actual_params = req.call_args.kwargs["params"]
            assert expected_params == actual_params


def test_search_nearby_rest_unset_required_fields():
    transport = transports.PlacesRestTransport(
        credentials=ga_credentials.AnonymousCredentials
    )

    unset_fields = transport.search_nearby._get_unset_required_fields({})
    assert set(unset_fields) == (set(()) & set(("locationRestriction",)))


@pytest.mark.parametrize("null_interceptor", [True, False])
def test_search_nearby_rest_interceptors(null_interceptor):
    transport = transports.PlacesRestTransport(
        credentials=ga_credentials.AnonymousCredentials(),
        interceptor=None if null_interceptor else transports.PlacesRestInterceptor(),
    )
    client = PlacesClient(transport=transport)
    with mock.patch.object(
        type(client.transport._session), "request"
    ) as req, mock.patch.object(
        path_template, "transcode"
    ) as transcode, mock.patch.object(
        transports.PlacesRestInterceptor, "post_search_nearby"
    ) as post, mock.patch.object(
        transports.PlacesRestInterceptor, "pre_search_nearby"
    ) as pre:
        pre.assert_not_called()
        post.assert_not_called()
        pb_message = places_service.SearchNearbyRequest.pb(
            places_service.SearchNearbyRequest()
        )
        transcode.return_value = {
            "method": "post",
            "uri": "my_uri",
            "body": pb_message,
            "query_params": pb_message,
        }

        req.return_value = Response()
        req.return_value.status_code = 200
        req.return_value.request = PreparedRequest()
        req.return_value._content = places_service.SearchNearbyResponse.to_json(
            places_service.SearchNearbyResponse()
        )

        request = places_service.SearchNearbyRequest()
        metadata = [
            ("key", "val"),
            ("cephalopod", "squid"),
        ]
        pre.return_value = request, metadata
        post.return_value = places_service.SearchNearbyResponse()

        client.search_nearby(
            request,
            metadata=[
                ("key", "val"),
                ("cephalopod", "squid"),
            ],
        )

        pre.assert_called_once()
        post.assert_called_once()


def test_search_nearby_rest_bad_request(
    transport: str = "rest", request_type=places_service.SearchNearbyRequest
):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # send a request that will satisfy transcoding
    request_init = {}
    request = request_type(**request_init)

    # Mock the http request call within the method and fake a BadRequest error.
    with mock.patch.object(Session, "request") as req, pytest.raises(
        core_exceptions.BadRequest
    ):
        # Wrap the value into a proper Response obj
        response_value = Response()
        response_value.status_code = 400
        response_value.request = Request()
        req.return_value = response_value
        client.search_nearby(request)


def test_search_nearby_rest_error():
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
    )


@pytest.mark.parametrize(
    "request_type",
    [
        places_service.SearchTextRequest,
        dict,
    ],
)
def test_search_text_rest(request_type):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )

    # send a request that will satisfy transcoding
    request_init = {}
    request = request_type(**request_init)

    # Mock the http request call within the method and fake a response.
    with mock.patch.object(type(client.transport._session), "request") as req:
        # Designate an appropriate value for the returned response.
        return_value = places_service.SearchTextResponse()

        # Wrap the value into a proper Response obj
        response_value = Response()
        response_value.status_code = 200
        # Convert return value to protobuf type
        return_value = places_service.SearchTextResponse.pb(return_value)
        json_return_value = json_format.MessageToJson(return_value)

        response_value._content = json_return_value.encode("UTF-8")
        req.return_value = response_value
        response = client.search_text(request)

    # Establish that the response is the type that we expect.
    assert isinstance(response, places_service.SearchTextResponse)


def test_search_text_rest_required_fields(
    request_type=places_service.SearchTextRequest,
):
    transport_class = transports.PlacesRestTransport

    request_init = {}
    request_init["text_query"] = ""
    request = request_type(**request_init)
    pb_request = request_type.pb(request)
    jsonified_request = json.loads(
        json_format.MessageToJson(pb_request, use_integers_for_enums=False)
    )

    # verify fields with default values are dropped

    unset_fields = transport_class(
        credentials=ga_credentials.AnonymousCredentials()
    ).search_text._get_unset_required_fields(jsonified_request)
    jsonified_request.update(unset_fields)

    # verify required fields with default values are now present

    jsonified_request["textQuery"] = "text_query_value"

    unset_fields = transport_class(
        credentials=ga_credentials.AnonymousCredentials()
    ).search_text._get_unset_required_fields(jsonified_request)
    jsonified_request.update(unset_fields)

    # verify required fields with non-default values are left alone
    assert "textQuery" in jsonified_request
    assert jsonified_request["textQuery"] == "text_query_value"

    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )
    request = request_type(**request_init)

    # Designate an appropriate value for the returned response.
    return_value = places_service.SearchTextResponse()
    # Mock the http request call within the method and fake a response.
    with mock.patch.object(Session, "request") as req:
        # We need to mock transcode() because providing default values
        # for required fields will fail the real version if the http_options
        # expect actual values for those fields.
        with mock.patch.object(path_template, "transcode") as transcode:
            # A uri without fields and an empty body will force all the
            # request fields to show up in the query_params.
            pb_request = request_type.pb(request)
            transcode_result = {
                "uri": "v1/sample_method",
                "method": "post",
                "query_params": pb_request,
            }
            transcode_result["body"] = pb_request
            transcode.return_value = transcode_result

            response_value = Response()
            response_value.status_code = 200

            # Convert return value to protobuf type
            return_value = places_service.SearchTextResponse.pb(return_value)
            json_return_value = json_format.MessageToJson(return_value)

            response_value._content = json_return_value.encode("UTF-8")
            req.return_value = response_value

            response = client.search_text(request)

            expected_params = [("$alt", "json;enum-encoding=int")]
            actual_params = req.call_args.kwargs["params"]
            assert expected_params == actual_params


def test_search_text_rest_unset_required_fields():
    transport = transports.PlacesRestTransport(
        credentials=ga_credentials.AnonymousCredentials
    )

    unset_fields = transport.search_text._get_unset_required_fields({})
    assert set(unset_fields) == (set(()) & set(("textQuery",)))


@pytest.mark.parametrize("null_interceptor", [True, False])
def test_search_text_rest_interceptors(null_interceptor):
    transport = transports.PlacesRestTransport(
        credentials=ga_credentials.AnonymousCredentials(),
        interceptor=None if null_interceptor else transports.PlacesRestInterceptor(),
    )
    client = PlacesClient(transport=transport)
    with mock.patch.object(
        type(client.transport._session), "request"
    ) as req, mock.patch.object(
        path_template, "transcode"
    ) as transcode, mock.patch.object(
        transports.PlacesRestInterceptor, "post_search_text"
    ) as post, mock.patch.object(
        transports.PlacesRestInterceptor, "pre_search_text"
    ) as pre:
        pre.assert_not_called()
        post.assert_not_called()
        pb_message = places_service.SearchTextRequest.pb(
            places_service.SearchTextRequest()
        )
        transcode.return_value = {
            "method": "post",
            "uri": "my_uri",
            "body": pb_message,
            "query_params": pb_message,
        }

        req.return_value = Response()
        req.return_value.status_code = 200
        req.return_value.request = PreparedRequest()
        req.return_value._content = places_service.SearchTextResponse.to_json(
            places_service.SearchTextResponse()
        )

        request = places_service.SearchTextRequest()
        metadata = [
            ("key", "val"),
            ("cephalopod", "squid"),
        ]
        pre.return_value = request, metadata
        post.return_value = places_service.SearchTextResponse()

        client.search_text(
            request,
            metadata=[
                ("key", "val"),
                ("cephalopod", "squid"),
            ],
        )

        pre.assert_called_once()
        post.assert_called_once()


def test_search_text_rest_bad_request(
    transport: str = "rest", request_type=places_service.SearchTextRequest
):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # send a request that will satisfy transcoding
    request_init = {}
    request = request_type(**request_init)

    # Mock the http request call within the method and fake a BadRequest error.
    with mock.patch.object(Session, "request") as req, pytest.raises(
        core_exceptions.BadRequest
    ):
        # Wrap the value into a proper Response obj
        response_value = Response()
        response_value.status_code = 400
        response_value.request = Request()
        req.return_value = response_value
        client.search_text(request)


def test_search_text_rest_error():
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
    )


@pytest.mark.parametrize(
    "request_type",
    [
        places_service.GetPhotoMediaRequest,
        dict,
    ],
)
def test_get_photo_media_rest(request_type):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )

    # send a request that will satisfy transcoding
    request_init = {"name": "places/sample1/photos/sample2/media"}
    request = request_type(**request_init)

    # Mock the http request call within the method and fake a response.
    with mock.patch.object(type(client.transport._session), "request") as req:
        # Designate an appropriate value for the returned response.
        return_value = places_service.PhotoMedia(
            name="name_value",
            photo_uri="photo_uri_value",
        )

        # Wrap the value into a proper Response obj
        response_value = Response()
        response_value.status_code = 200
        # Convert return value to protobuf type
        return_value = places_service.PhotoMedia.pb(return_value)
        json_return_value = json_format.MessageToJson(return_value)

        response_value._content = json_return_value.encode("UTF-8")
        req.return_value = response_value
        response = client.get_photo_media(request)

    # Establish that the response is the type that we expect.
    assert isinstance(response, places_service.PhotoMedia)
    assert response.name == "name_value"
    assert response.photo_uri == "photo_uri_value"


def test_get_photo_media_rest_required_fields(
    request_type=places_service.GetPhotoMediaRequest,
):
    transport_class = transports.PlacesRestTransport

    request_init = {}
    request_init["name"] = ""
    request = request_type(**request_init)
    pb_request = request_type.pb(request)
    jsonified_request = json.loads(
        json_format.MessageToJson(pb_request, use_integers_for_enums=False)
    )

    # verify fields with default values are dropped

    unset_fields = transport_class(
        credentials=ga_credentials.AnonymousCredentials()
    ).get_photo_media._get_unset_required_fields(jsonified_request)
    jsonified_request.update(unset_fields)

    # verify required fields with default values are now present

    jsonified_request["name"] = "name_value"

    unset_fields = transport_class(
        credentials=ga_credentials.AnonymousCredentials()
    ).get_photo_media._get_unset_required_fields(jsonified_request)
    # Check that path parameters and body parameters are not mixing in.
    assert not set(unset_fields) - set(
        (
            "max_height_px",
            "max_width_px",
            "skip_http_redirect",
        )
    )
    jsonified_request.update(unset_fields)

    # verify required fields with non-default values are left alone
    assert "name" in jsonified_request
    assert jsonified_request["name"] == "name_value"

    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )
    request = request_type(**request_init)

    # Designate an appropriate value for the returned response.
    return_value = places_service.PhotoMedia()
    # Mock the http request call within the method and fake a response.
    with mock.patch.object(Session, "request") as req:
        # We need to mock transcode() because providing default values
        # for required fields will fail the real version if the http_options
        # expect actual values for those fields.
        with mock.patch.object(path_template, "transcode") as transcode:
            # A uri without fields and an empty body will force all the
            # request fields to show up in the query_params.
            pb_request = request_type.pb(request)
            transcode_result = {
                "uri": "v1/sample_method",
                "method": "get",
                "query_params": pb_request,
            }
            transcode.return_value = transcode_result

            response_value = Response()
            response_value.status_code = 200

            # Convert return value to protobuf type
            return_value = places_service.PhotoMedia.pb(return_value)
            json_return_value = json_format.MessageToJson(return_value)

            response_value._content = json_return_value.encode("UTF-8")
            req.return_value = response_value

            response = client.get_photo_media(request)

            expected_params = [("$alt", "json;enum-encoding=int")]
            actual_params = req.call_args.kwargs["params"]
            assert expected_params == actual_params


def test_get_photo_media_rest_unset_required_fields():
    transport = transports.PlacesRestTransport(
        credentials=ga_credentials.AnonymousCredentials
    )

    unset_fields = transport.get_photo_media._get_unset_required_fields({})
    assert set(unset_fields) == (
        set(
            (
                "maxHeightPx",
                "maxWidthPx",
                "skipHttpRedirect",
            )
        )
        & set(("name",))
    )


@pytest.mark.parametrize("null_interceptor", [True, False])
def test_get_photo_media_rest_interceptors(null_interceptor):
    transport = transports.PlacesRestTransport(
        credentials=ga_credentials.AnonymousCredentials(),
        interceptor=None if null_interceptor else transports.PlacesRestInterceptor(),
    )
    client = PlacesClient(transport=transport)
    with mock.patch.object(
        type(client.transport._session), "request"
    ) as req, mock.patch.object(
        path_template, "transcode"
    ) as transcode, mock.patch.object(
        transports.PlacesRestInterceptor, "post_get_photo_media"
    ) as post, mock.patch.object(
        transports.PlacesRestInterceptor, "pre_get_photo_media"
    ) as pre:
        pre.assert_not_called()
        post.assert_not_called()
        pb_message = places_service.GetPhotoMediaRequest.pb(
            places_service.GetPhotoMediaRequest()
        )
        transcode.return_value = {
            "method": "post",
            "uri": "my_uri",
            "body": pb_message,
            "query_params": pb_message,
        }

        req.return_value = Response()
        req.return_value.status_code = 200
        req.return_value.request = PreparedRequest()
        req.return_value._content = places_service.PhotoMedia.to_json(
            places_service.PhotoMedia()
        )

        request = places_service.GetPhotoMediaRequest()
        metadata = [
            ("key", "val"),
            ("cephalopod", "squid"),
        ]
        pre.return_value = request, metadata
        post.return_value = places_service.PhotoMedia()

        client.get_photo_media(
            request,
            metadata=[
                ("key", "val"),
                ("cephalopod", "squid"),
            ],
        )

        pre.assert_called_once()
        post.assert_called_once()


def test_get_photo_media_rest_bad_request(
    transport: str = "rest", request_type=places_service.GetPhotoMediaRequest
):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # send a request that will satisfy transcoding
    request_init = {"name": "places/sample1/photos/sample2/media"}
    request = request_type(**request_init)

    # Mock the http request call within the method and fake a BadRequest error.
    with mock.patch.object(Session, "request") as req, pytest.raises(
        core_exceptions.BadRequest
    ):
        # Wrap the value into a proper Response obj
        response_value = Response()
        response_value.status_code = 400
        response_value.request = Request()
        req.return_value = response_value
        client.get_photo_media(request)


def test_get_photo_media_rest_flattened():
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )

    # Mock the http request call within the method and fake a response.
    with mock.patch.object(type(client.transport._session), "request") as req:
        # Designate an appropriate value for the returned response.
        return_value = places_service.PhotoMedia()

        # get arguments that satisfy an http rule for this method
        sample_request = {"name": "places/sample1/photos/sample2/media"}

        # get truthy value for each flattened field
        mock_args = dict(
            name="name_value",
        )
        mock_args.update(sample_request)

        # Wrap the value into a proper Response obj
        response_value = Response()
        response_value.status_code = 200
        # Convert return value to protobuf type
        return_value = places_service.PhotoMedia.pb(return_value)
        json_return_value = json_format.MessageToJson(return_value)
        response_value._content = json_return_value.encode("UTF-8")
        req.return_value = response_value

        client.get_photo_media(**mock_args)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(req.mock_calls) == 1
        _, args, _ = req.mock_calls[0]
        assert path_template.validate(
            "%s/v1/{name=places/*/photos/*/media}" % client.transport._host, args[1]
        )


def test_get_photo_media_rest_flattened_error(transport: str = "rest"):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.get_photo_media(
            places_service.GetPhotoMediaRequest(),
            name="name_value",
        )


def test_get_photo_media_rest_error():
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
    )


@pytest.mark.parametrize(
    "request_type",
    [
        places_service.GetPlaceRequest,
        dict,
    ],
)
def test_get_place_rest(request_type):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )

    # send a request that will satisfy transcoding
    request_init = {"name": "places/sample1"}
    request = request_type(**request_init)

    # Mock the http request call within the method and fake a response.
    with mock.patch.object(type(client.transport._session), "request") as req:
        # Designate an appropriate value for the returned response.
        return_value = place.Place(
            name="name_value",
            id="id_value",
            types=["types_value"],
            primary_type="primary_type_value",
            national_phone_number="national_phone_number_value",
            international_phone_number="international_phone_number_value",
            formatted_address="formatted_address_value",
            short_formatted_address="short_formatted_address_value",
            rating=0.645,
            google_maps_uri="google_maps_uri_value",
            website_uri="website_uri_value",
            utc_offset_minutes=1942,
            adr_format_address="adr_format_address_value",
            business_status=place.Place.BusinessStatus.OPERATIONAL,
            price_level=place.PriceLevel.PRICE_LEVEL_FREE,
            user_rating_count=1835,
            icon_mask_base_uri="icon_mask_base_uri_value",
            icon_background_color="icon_background_color_value",
            takeout=True,
            delivery=True,
            dine_in=True,
            curbside_pickup=True,
            reservable=True,
            serves_breakfast=True,
            serves_lunch=True,
            serves_dinner=True,
            serves_beer=True,
            serves_wine=True,
            serves_brunch=True,
            serves_vegetarian_food=True,
            outdoor_seating=True,
            live_music=True,
            menu_for_children=True,
            serves_cocktails=True,
            serves_dessert=True,
            serves_coffee=True,
            good_for_children=True,
            allows_dogs=True,
            restroom=True,
            good_for_groups=True,
            good_for_watching_sports=True,
        )

        # Wrap the value into a proper Response obj
        response_value = Response()
        response_value.status_code = 200
        # Convert return value to protobuf type
        return_value = place.Place.pb(return_value)
        json_return_value = json_format.MessageToJson(return_value)

        response_value._content = json_return_value.encode("UTF-8")
        req.return_value = response_value
        response = client.get_place(request)

    # Establish that the response is the type that we expect.
    assert isinstance(response, place.Place)
    assert response.name == "name_value"
    assert response.id == "id_value"
    assert response.types == ["types_value"]
    assert response.primary_type == "primary_type_value"
    assert response.national_phone_number == "national_phone_number_value"
    assert response.international_phone_number == "international_phone_number_value"
    assert response.formatted_address == "formatted_address_value"
    assert response.short_formatted_address == "short_formatted_address_value"
    assert math.isclose(response.rating, 0.645, rel_tol=1e-6)
    assert response.google_maps_uri == "google_maps_uri_value"
    assert response.website_uri == "website_uri_value"
    assert response.utc_offset_minutes == 1942
    assert response.adr_format_address == "adr_format_address_value"
    assert response.business_status == place.Place.BusinessStatus.OPERATIONAL
    assert response.price_level == place.PriceLevel.PRICE_LEVEL_FREE
    assert response.user_rating_count == 1835
    assert response.icon_mask_base_uri == "icon_mask_base_uri_value"
    assert response.icon_background_color == "icon_background_color_value"
    assert response.takeout is True
    assert response.delivery is True
    assert response.dine_in is True
    assert response.curbside_pickup is True
    assert response.reservable is True
    assert response.serves_breakfast is True
    assert response.serves_lunch is True
    assert response.serves_dinner is True
    assert response.serves_beer is True
    assert response.serves_wine is True
    assert response.serves_brunch is True
    assert response.serves_vegetarian_food is True
    assert response.outdoor_seating is True
    assert response.live_music is True
    assert response.menu_for_children is True
    assert response.serves_cocktails is True
    assert response.serves_dessert is True
    assert response.serves_coffee is True
    assert response.good_for_children is True
    assert response.allows_dogs is True
    assert response.restroom is True
    assert response.good_for_groups is True
    assert response.good_for_watching_sports is True


def test_get_place_rest_required_fields(request_type=places_service.GetPlaceRequest):
    transport_class = transports.PlacesRestTransport

    request_init = {}
    request_init["name"] = ""
    request = request_type(**request_init)
    pb_request = request_type.pb(request)
    jsonified_request = json.loads(
        json_format.MessageToJson(pb_request, use_integers_for_enums=False)
    )

    # verify fields with default values are dropped

    unset_fields = transport_class(
        credentials=ga_credentials.AnonymousCredentials()
    ).get_place._get_unset_required_fields(jsonified_request)
    jsonified_request.update(unset_fields)

    # verify required fields with default values are now present

    jsonified_request["name"] = "name_value"

    unset_fields = transport_class(
        credentials=ga_credentials.AnonymousCredentials()
    ).get_place._get_unset_required_fields(jsonified_request)
    # Check that path parameters and body parameters are not mixing in.
    assert not set(unset_fields) - set(
        (
            "language_code",
            "region_code",
            "session_token",
        )
    )
    jsonified_request.update(unset_fields)

    # verify required fields with non-default values are left alone
    assert "name" in jsonified_request
    assert jsonified_request["name"] == "name_value"

    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )
    request = request_type(**request_init)

    # Designate an appropriate value for the returned response.
    return_value = place.Place()
    # Mock the http request call within the method and fake a response.
    with mock.patch.object(Session, "request") as req:
        # We need to mock transcode() because providing default values
        # for required fields will fail the real version if the http_options
        # expect actual values for those fields.
        with mock.patch.object(path_template, "transcode") as transcode:
            # A uri without fields and an empty body will force all the
            # request fields to show up in the query_params.
            pb_request = request_type.pb(request)
            transcode_result = {
                "uri": "v1/sample_method",
                "method": "get",
                "query_params": pb_request,
            }
            transcode.return_value = transcode_result

            response_value = Response()
            response_value.status_code = 200

            # Convert return value to protobuf type
            return_value = place.Place.pb(return_value)
            json_return_value = json_format.MessageToJson(return_value)

            response_value._content = json_return_value.encode("UTF-8")
            req.return_value = response_value

            response = client.get_place(request)

            expected_params = [("$alt", "json;enum-encoding=int")]
            actual_params = req.call_args.kwargs["params"]
            assert expected_params == actual_params


def test_get_place_rest_unset_required_fields():
    transport = transports.PlacesRestTransport(
        credentials=ga_credentials.AnonymousCredentials
    )

    unset_fields = transport.get_place._get_unset_required_fields({})
    assert set(unset_fields) == (
        set(
            (
                "languageCode",
                "regionCode",
                "sessionToken",
            )
        )
        & set(("name",))
    )


@pytest.mark.parametrize("null_interceptor", [True, False])
def test_get_place_rest_interceptors(null_interceptor):
    transport = transports.PlacesRestTransport(
        credentials=ga_credentials.AnonymousCredentials(),
        interceptor=None if null_interceptor else transports.PlacesRestInterceptor(),
    )
    client = PlacesClient(transport=transport)
    with mock.patch.object(
        type(client.transport._session), "request"
    ) as req, mock.patch.object(
        path_template, "transcode"
    ) as transcode, mock.patch.object(
        transports.PlacesRestInterceptor, "post_get_place"
    ) as post, mock.patch.object(
        transports.PlacesRestInterceptor, "pre_get_place"
    ) as pre:
        pre.assert_not_called()
        post.assert_not_called()
        pb_message = places_service.GetPlaceRequest.pb(places_service.GetPlaceRequest())
        transcode.return_value = {
            "method": "post",
            "uri": "my_uri",
            "body": pb_message,
            "query_params": pb_message,
        }

        req.return_value = Response()
        req.return_value.status_code = 200
        req.return_value.request = PreparedRequest()
        req.return_value._content = place.Place.to_json(place.Place())

        request = places_service.GetPlaceRequest()
        metadata = [
            ("key", "val"),
            ("cephalopod", "squid"),
        ]
        pre.return_value = request, metadata
        post.return_value = place.Place()

        client.get_place(
            request,
            metadata=[
                ("key", "val"),
                ("cephalopod", "squid"),
            ],
        )

        pre.assert_called_once()
        post.assert_called_once()


def test_get_place_rest_bad_request(
    transport: str = "rest", request_type=places_service.GetPlaceRequest
):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # send a request that will satisfy transcoding
    request_init = {"name": "places/sample1"}
    request = request_type(**request_init)

    # Mock the http request call within the method and fake a BadRequest error.
    with mock.patch.object(Session, "request") as req, pytest.raises(
        core_exceptions.BadRequest
    ):
        # Wrap the value into a proper Response obj
        response_value = Response()
        response_value.status_code = 400
        response_value.request = Request()
        req.return_value = response_value
        client.get_place(request)


def test_get_place_rest_flattened():
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )

    # Mock the http request call within the method and fake a response.
    with mock.patch.object(type(client.transport._session), "request") as req:
        # Designate an appropriate value for the returned response.
        return_value = place.Place()

        # get arguments that satisfy an http rule for this method
        sample_request = {"name": "places/sample1"}

        # get truthy value for each flattened field
        mock_args = dict(
            name="name_value",
        )
        mock_args.update(sample_request)

        # Wrap the value into a proper Response obj
        response_value = Response()
        response_value.status_code = 200
        # Convert return value to protobuf type
        return_value = place.Place.pb(return_value)
        json_return_value = json_format.MessageToJson(return_value)
        response_value._content = json_return_value.encode("UTF-8")
        req.return_value = response_value

        client.get_place(**mock_args)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(req.mock_calls) == 1
        _, args, _ = req.mock_calls[0]
        assert path_template.validate(
            "%s/v1/{name=places/*}" % client.transport._host, args[1]
        )


def test_get_place_rest_flattened_error(transport: str = "rest"):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.get_place(
            places_service.GetPlaceRequest(),
            name="name_value",
        )


def test_get_place_rest_error():
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
    )


@pytest.mark.parametrize(
    "request_type",
    [
        places_service.AutocompletePlacesRequest,
        dict,
    ],
)
def test_autocomplete_places_rest(request_type):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )

    # send a request that will satisfy transcoding
    request_init = {}
    request = request_type(**request_init)

    # Mock the http request call within the method and fake a response.
    with mock.patch.object(type(client.transport._session), "request") as req:
        # Designate an appropriate value for the returned response.
        return_value = places_service.AutocompletePlacesResponse()

        # Wrap the value into a proper Response obj
        response_value = Response()
        response_value.status_code = 200
        # Convert return value to protobuf type
        return_value = places_service.AutocompletePlacesResponse.pb(return_value)
        json_return_value = json_format.MessageToJson(return_value)

        response_value._content = json_return_value.encode("UTF-8")
        req.return_value = response_value
        response = client.autocomplete_places(request)

    # Establish that the response is the type that we expect.
    assert isinstance(response, places_service.AutocompletePlacesResponse)


def test_autocomplete_places_rest_required_fields(
    request_type=places_service.AutocompletePlacesRequest,
):
    transport_class = transports.PlacesRestTransport

    request_init = {}
    request_init["input"] = ""
    request = request_type(**request_init)
    pb_request = request_type.pb(request)
    jsonified_request = json.loads(
        json_format.MessageToJson(pb_request, use_integers_for_enums=False)
    )

    # verify fields with default values are dropped

    unset_fields = transport_class(
        credentials=ga_credentials.AnonymousCredentials()
    ).autocomplete_places._get_unset_required_fields(jsonified_request)
    jsonified_request.update(unset_fields)

    # verify required fields with default values are now present

    jsonified_request["input"] = "input_value"

    unset_fields = transport_class(
        credentials=ga_credentials.AnonymousCredentials()
    ).autocomplete_places._get_unset_required_fields(jsonified_request)
    jsonified_request.update(unset_fields)

    # verify required fields with non-default values are left alone
    assert "input" in jsonified_request
    assert jsonified_request["input"] == "input_value"

    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )
    request = request_type(**request_init)

    # Designate an appropriate value for the returned response.
    return_value = places_service.AutocompletePlacesResponse()
    # Mock the http request call within the method and fake a response.
    with mock.patch.object(Session, "request") as req:
        # We need to mock transcode() because providing default values
        # for required fields will fail the real version if the http_options
        # expect actual values for those fields.
        with mock.patch.object(path_template, "transcode") as transcode:
            # A uri without fields and an empty body will force all the
            # request fields to show up in the query_params.
            pb_request = request_type.pb(request)
            transcode_result = {
                "uri": "v1/sample_method",
                "method": "post",
                "query_params": pb_request,
            }
            transcode_result["body"] = pb_request
            transcode.return_value = transcode_result

            response_value = Response()
            response_value.status_code = 200

            # Convert return value to protobuf type
            return_value = places_service.AutocompletePlacesResponse.pb(return_value)
            json_return_value = json_format.MessageToJson(return_value)

            response_value._content = json_return_value.encode("UTF-8")
            req.return_value = response_value

            response = client.autocomplete_places(request)

            expected_params = [("$alt", "json;enum-encoding=int")]
            actual_params = req.call_args.kwargs["params"]
            assert expected_params == actual_params


def test_autocomplete_places_rest_unset_required_fields():
    transport = transports.PlacesRestTransport(
        credentials=ga_credentials.AnonymousCredentials
    )

    unset_fields = transport.autocomplete_places._get_unset_required_fields({})
    assert set(unset_fields) == (set(()) & set(("input",)))


@pytest.mark.parametrize("null_interceptor", [True, False])
def test_autocomplete_places_rest_interceptors(null_interceptor):
    transport = transports.PlacesRestTransport(
        credentials=ga_credentials.AnonymousCredentials(),
        interceptor=None if null_interceptor else transports.PlacesRestInterceptor(),
    )
    client = PlacesClient(transport=transport)
    with mock.patch.object(
        type(client.transport._session), "request"
    ) as req, mock.patch.object(
        path_template, "transcode"
    ) as transcode, mock.patch.object(
        transports.PlacesRestInterceptor, "post_autocomplete_places"
    ) as post, mock.patch.object(
        transports.PlacesRestInterceptor, "pre_autocomplete_places"
    ) as pre:
        pre.assert_not_called()
        post.assert_not_called()
        pb_message = places_service.AutocompletePlacesRequest.pb(
            places_service.AutocompletePlacesRequest()
        )
        transcode.return_value = {
            "method": "post",
            "uri": "my_uri",
            "body": pb_message,
            "query_params": pb_message,
        }

        req.return_value = Response()
        req.return_value.status_code = 200
        req.return_value.request = PreparedRequest()
        req.return_value._content = places_service.AutocompletePlacesResponse.to_json(
            places_service.AutocompletePlacesResponse()
        )

        request = places_service.AutocompletePlacesRequest()
        metadata = [
            ("key", "val"),
            ("cephalopod", "squid"),
        ]
        pre.return_value = request, metadata
        post.return_value = places_service.AutocompletePlacesResponse()

        client.autocomplete_places(
            request,
            metadata=[
                ("key", "val"),
                ("cephalopod", "squid"),
            ],
        )

        pre.assert_called_once()
        post.assert_called_once()


def test_autocomplete_places_rest_bad_request(
    transport: str = "rest", request_type=places_service.AutocompletePlacesRequest
):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # send a request that will satisfy transcoding
    request_init = {}
    request = request_type(**request_init)

    # Mock the http request call within the method and fake a BadRequest error.
    with mock.patch.object(Session, "request") as req, pytest.raises(
        core_exceptions.BadRequest
    ):
        # Wrap the value into a proper Response obj
        response_value = Response()
        response_value.status_code = 400
        response_value.request = Request()
        req.return_value = response_value
        client.autocomplete_places(request)


def test_autocomplete_places_rest_error():
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
    )


def test_credentials_transport_error():
    # It is an error to provide credentials and a transport instance.
    transport = transports.PlacesGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    with pytest.raises(ValueError):
        client = PlacesClient(
            credentials=ga_credentials.AnonymousCredentials(),
            transport=transport,
        )

    # It is an error to provide a credentials file and a transport instance.
    transport = transports.PlacesGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    with pytest.raises(ValueError):
        client = PlacesClient(
            client_options={"credentials_file": "credentials.json"},
            transport=transport,
        )

    # It is an error to provide an api_key and a transport instance.
    transport = transports.PlacesGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    options = client_options.ClientOptions()
    options.api_key = "api_key"
    with pytest.raises(ValueError):
        client = PlacesClient(
            client_options=options,
            transport=transport,
        )

    # It is an error to provide an api_key and a credential.
    options = client_options.ClientOptions()
    options.api_key = "api_key"
    with pytest.raises(ValueError):
        client = PlacesClient(
            client_options=options, credentials=ga_credentials.AnonymousCredentials()
        )

    # It is an error to provide scopes and a transport instance.
    transport = transports.PlacesGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    with pytest.raises(ValueError):
        client = PlacesClient(
            client_options={"scopes": ["1", "2"]},
            transport=transport,
        )


def test_transport_instance():
    # A client may be instantiated with a custom transport instance.
    transport = transports.PlacesGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    client = PlacesClient(transport=transport)
    assert client.transport is transport


def test_transport_get_channel():
    # A client may be instantiated with a custom transport instance.
    transport = transports.PlacesGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    channel = transport.grpc_channel
    assert channel

    transport = transports.PlacesGrpcAsyncIOTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    channel = transport.grpc_channel
    assert channel


@pytest.mark.parametrize(
    "transport_class",
    [
        transports.PlacesGrpcTransport,
        transports.PlacesGrpcAsyncIOTransport,
        transports.PlacesRestTransport,
    ],
)
def test_transport_adc(transport_class):
    # Test default credentials are used if not provided.
    with mock.patch.object(google.auth, "default") as adc:
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport_class()
        adc.assert_called_once()


@pytest.mark.parametrize(
    "transport_name",
    [
        "grpc",
        "rest",
    ],
)
def test_transport_kind(transport_name):
    transport = PlacesClient.get_transport_class(transport_name)(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    assert transport.kind == transport_name


def test_transport_grpc_default():
    # A client should use the gRPC transport by default.
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    assert isinstance(
        client.transport,
        transports.PlacesGrpcTransport,
    )


def test_places_base_transport_error():
    # Passing both a credentials object and credentials_file should raise an error
    with pytest.raises(core_exceptions.DuplicateCredentialArgs):
        transport = transports.PlacesTransport(
            credentials=ga_credentials.AnonymousCredentials(),
            credentials_file="credentials.json",
        )


def test_places_base_transport():
    # Instantiate the base transport.
    with mock.patch(
        "google.maps.places_v1.services.places.transports.PlacesTransport.__init__"
    ) as Transport:
        Transport.return_value = None
        transport = transports.PlacesTransport(
            credentials=ga_credentials.AnonymousCredentials(),
        )

    # Every method on the transport should just blindly
    # raise NotImplementedError.
    methods = (
        "search_nearby",
        "search_text",
        "get_photo_media",
        "get_place",
        "autocomplete_places",
    )
    for method in methods:
        with pytest.raises(NotImplementedError):
            getattr(transport, method)(request=object())

    with pytest.raises(NotImplementedError):
        transport.close()

    # Catch all for all remaining methods and properties
    remainder = [
        "kind",
    ]
    for r in remainder:
        with pytest.raises(NotImplementedError):
            getattr(transport, r)()


def test_places_base_transport_with_credentials_file():
    # Instantiate the base transport with a credentials file
    with mock.patch.object(
        google.auth, "load_credentials_from_file", autospec=True
    ) as load_creds, mock.patch(
        "google.maps.places_v1.services.places.transports.PlacesTransport._prep_wrapped_messages"
    ) as Transport:
        Transport.return_value = None
        load_creds.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport = transports.PlacesTransport(
            credentials_file="credentials.json",
            quota_project_id="octopus",
        )
        load_creds.assert_called_once_with(
            "credentials.json",
            scopes=None,
            default_scopes=(),
            quota_project_id="octopus",
        )


def test_places_base_transport_with_adc():
    # Test the default credentials are used if credentials and credentials_file are None.
    with mock.patch.object(google.auth, "default", autospec=True) as adc, mock.patch(
        "google.maps.places_v1.services.places.transports.PlacesTransport._prep_wrapped_messages"
    ) as Transport:
        Transport.return_value = None
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport = transports.PlacesTransport()
        adc.assert_called_once()


def test_places_auth_adc():
    # If no credentials are provided, we should use ADC credentials.
    with mock.patch.object(google.auth, "default", autospec=True) as adc:
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        PlacesClient()
        adc.assert_called_once_with(
            scopes=None,
            default_scopes=(),
            quota_project_id=None,
        )


@pytest.mark.parametrize(
    "transport_class",
    [
        transports.PlacesGrpcTransport,
        transports.PlacesGrpcAsyncIOTransport,
    ],
)
def test_places_transport_auth_adc(transport_class):
    # If credentials and host are not provided, the transport class should use
    # ADC credentials.
    with mock.patch.object(google.auth, "default", autospec=True) as adc:
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport_class(quota_project_id="octopus", scopes=["1", "2"])
        adc.assert_called_once_with(
            scopes=["1", "2"],
            default_scopes=(),
            quota_project_id="octopus",
        )


@pytest.mark.parametrize(
    "transport_class",
    [
        transports.PlacesGrpcTransport,
        transports.PlacesGrpcAsyncIOTransport,
        transports.PlacesRestTransport,
    ],
)
def test_places_transport_auth_gdch_credentials(transport_class):
    host = "https://language.com"
    api_audience_tests = [None, "https://language2.com"]
    api_audience_expect = [host, "https://language2.com"]
    for t, e in zip(api_audience_tests, api_audience_expect):
        with mock.patch.object(google.auth, "default", autospec=True) as adc:
            gdch_mock = mock.MagicMock()
            type(gdch_mock).with_gdch_audience = mock.PropertyMock(
                return_value=gdch_mock
            )
            adc.return_value = (gdch_mock, None)
            transport_class(host=host, api_audience=t)
            gdch_mock.with_gdch_audience.assert_called_once_with(e)


@pytest.mark.parametrize(
    "transport_class,grpc_helpers",
    [
        (transports.PlacesGrpcTransport, grpc_helpers),
        (transports.PlacesGrpcAsyncIOTransport, grpc_helpers_async),
    ],
)
def test_places_transport_create_channel(transport_class, grpc_helpers):
    # If credentials and host are not provided, the transport class should use
    # ADC credentials.
    with mock.patch.object(
        google.auth, "default", autospec=True
    ) as adc, mock.patch.object(
        grpc_helpers, "create_channel", autospec=True
    ) as create_channel:
        creds = ga_credentials.AnonymousCredentials()
        adc.return_value = (creds, None)
        transport_class(quota_project_id="octopus", scopes=["1", "2"])

        create_channel.assert_called_with(
            "places.googleapis.com:443",
            credentials=creds,
            credentials_file=None,
            quota_project_id="octopus",
            default_scopes=(),
            scopes=["1", "2"],
            default_host="places.googleapis.com",
            ssl_credentials=None,
            options=[
                ("grpc.max_send_message_length", -1),
                ("grpc.max_receive_message_length", -1),
            ],
        )


@pytest.mark.parametrize(
    "transport_class",
    [transports.PlacesGrpcTransport, transports.PlacesGrpcAsyncIOTransport],
)
def test_places_grpc_transport_client_cert_source_for_mtls(transport_class):
    cred = ga_credentials.AnonymousCredentials()

    # Check ssl_channel_credentials is used if provided.
    with mock.patch.object(transport_class, "create_channel") as mock_create_channel:
        mock_ssl_channel_creds = mock.Mock()
        transport_class(
            host="squid.clam.whelk",
            credentials=cred,
            ssl_channel_credentials=mock_ssl_channel_creds,
        )
        mock_create_channel.assert_called_once_with(
            "squid.clam.whelk:443",
            credentials=cred,
            credentials_file=None,
            scopes=None,
            ssl_credentials=mock_ssl_channel_creds,
            quota_project_id=None,
            options=[
                ("grpc.max_send_message_length", -1),
                ("grpc.max_receive_message_length", -1),
            ],
        )

    # Check if ssl_channel_credentials is not provided, then client_cert_source_for_mtls
    # is used.
    with mock.patch.object(transport_class, "create_channel", return_value=mock.Mock()):
        with mock.patch("grpc.ssl_channel_credentials") as mock_ssl_cred:
            transport_class(
                credentials=cred,
                client_cert_source_for_mtls=client_cert_source_callback,
            )
            expected_cert, expected_key = client_cert_source_callback()
            mock_ssl_cred.assert_called_once_with(
                certificate_chain=expected_cert, private_key=expected_key
            )


def test_places_http_transport_client_cert_source_for_mtls():
    cred = ga_credentials.AnonymousCredentials()
    with mock.patch(
        "google.auth.transport.requests.AuthorizedSession.configure_mtls_channel"
    ) as mock_configure_mtls_channel:
        transports.PlacesRestTransport(
            credentials=cred, client_cert_source_for_mtls=client_cert_source_callback
        )
        mock_configure_mtls_channel.assert_called_once_with(client_cert_source_callback)


@pytest.mark.parametrize(
    "transport_name",
    [
        "grpc",
        "grpc_asyncio",
        "rest",
    ],
)
def test_places_host_no_port(transport_name):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        client_options=client_options.ClientOptions(
            api_endpoint="places.googleapis.com"
        ),
        transport=transport_name,
    )
    assert client.transport._host == (
        "places.googleapis.com:443"
        if transport_name in ["grpc", "grpc_asyncio"]
        else "https://places.googleapis.com"
    )


@pytest.mark.parametrize(
    "transport_name",
    [
        "grpc",
        "grpc_asyncio",
        "rest",
    ],
)
def test_places_host_with_port(transport_name):
    client = PlacesClient(
        credentials=ga_credentials.AnonymousCredentials(),
        client_options=client_options.ClientOptions(
            api_endpoint="places.googleapis.com:8000"
        ),
        transport=transport_name,
    )
    assert client.transport._host == (
        "places.googleapis.com:8000"
        if transport_name in ["grpc", "grpc_asyncio"]
        else "https://places.googleapis.com:8000"
    )


@pytest.mark.parametrize(
    "transport_name",
    [
        "rest",
    ],
)
def test_places_client_transport_session_collision(transport_name):
    creds1 = ga_credentials.AnonymousCredentials()
    creds2 = ga_credentials.AnonymousCredentials()
    client1 = PlacesClient(
        credentials=creds1,
        transport=transport_name,
    )
    client2 = PlacesClient(
        credentials=creds2,
        transport=transport_name,
    )
    session1 = client1.transport.search_nearby._session
    session2 = client2.transport.search_nearby._session
    assert session1 != session2
    session1 = client1.transport.search_text._session
    session2 = client2.transport.search_text._session
    assert session1 != session2
    session1 = client1.transport.get_photo_media._session
    session2 = client2.transport.get_photo_media._session
    assert session1 != session2
    session1 = client1.transport.get_place._session
    session2 = client2.transport.get_place._session
    assert session1 != session2
    session1 = client1.transport.autocomplete_places._session
    session2 = client2.transport.autocomplete_places._session
    assert session1 != session2


def test_places_grpc_transport_channel():
    channel = grpc.secure_channel("http://localhost/", grpc.local_channel_credentials())

    # Check that channel is used if provided.
    transport = transports.PlacesGrpcTransport(
        host="squid.clam.whelk",
        channel=channel,
    )
    assert transport.grpc_channel == channel
    assert transport._host == "squid.clam.whelk:443"
    assert transport._ssl_channel_credentials == None


def test_places_grpc_asyncio_transport_channel():
    channel = aio.secure_channel("http://localhost/", grpc.local_channel_credentials())

    # Check that channel is used if provided.
    transport = transports.PlacesGrpcAsyncIOTransport(
        host="squid.clam.whelk",
        channel=channel,
    )
    assert transport.grpc_channel == channel
    assert transport._host == "squid.clam.whelk:443"
    assert transport._ssl_channel_credentials == None


# Remove this test when deprecated arguments (api_mtls_endpoint, client_cert_source) are
# removed from grpc/grpc_asyncio transport constructor.
@pytest.mark.parametrize(
    "transport_class",
    [transports.PlacesGrpcTransport, transports.PlacesGrpcAsyncIOTransport],
)
def test_places_transport_channel_mtls_with_client_cert_source(transport_class):
    with mock.patch(
        "grpc.ssl_channel_credentials", autospec=True
    ) as grpc_ssl_channel_cred:
        with mock.patch.object(
            transport_class, "create_channel"
        ) as grpc_create_channel:
            mock_ssl_cred = mock.Mock()
            grpc_ssl_channel_cred.return_value = mock_ssl_cred

            mock_grpc_channel = mock.Mock()
            grpc_create_channel.return_value = mock_grpc_channel

            cred = ga_credentials.AnonymousCredentials()
            with pytest.warns(DeprecationWarning):
                with mock.patch.object(google.auth, "default") as adc:
                    adc.return_value = (cred, None)
                    transport = transport_class(
                        host="squid.clam.whelk",
                        api_mtls_endpoint="mtls.squid.clam.whelk",
                        client_cert_source=client_cert_source_callback,
                    )
                    adc.assert_called_once()

            grpc_ssl_channel_cred.assert_called_once_with(
                certificate_chain=b"cert bytes", private_key=b"key bytes"
            )
            grpc_create_channel.assert_called_once_with(
                "mtls.squid.clam.whelk:443",
                credentials=cred,
                credentials_file=None,
                scopes=None,
                ssl_credentials=mock_ssl_cred,
                quota_project_id=None,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )
            assert transport.grpc_channel == mock_grpc_channel
            assert transport._ssl_channel_credentials == mock_ssl_cred


# Remove this test when deprecated arguments (api_mtls_endpoint, client_cert_source) are
# removed from grpc/grpc_asyncio transport constructor.
@pytest.mark.parametrize(
    "transport_class",
    [transports.PlacesGrpcTransport, transports.PlacesGrpcAsyncIOTransport],
)
def test_places_transport_channel_mtls_with_adc(transport_class):
    mock_ssl_cred = mock.Mock()
    with mock.patch.multiple(
        "google.auth.transport.grpc.SslCredentials",
        __init__=mock.Mock(return_value=None),
        ssl_credentials=mock.PropertyMock(return_value=mock_ssl_cred),
    ):
        with mock.patch.object(
            transport_class, "create_channel"
        ) as grpc_create_channel:
            mock_grpc_channel = mock.Mock()
            grpc_create_channel.return_value = mock_grpc_channel
            mock_cred = mock.Mock()

            with pytest.warns(DeprecationWarning):
                transport = transport_class(
                    host="squid.clam.whelk",
                    credentials=mock_cred,
                    api_mtls_endpoint="mtls.squid.clam.whelk",
                    client_cert_source=None,
                )

            grpc_create_channel.assert_called_once_with(
                "mtls.squid.clam.whelk:443",
                credentials=mock_cred,
                credentials_file=None,
                scopes=None,
                ssl_credentials=mock_ssl_cred,
                quota_project_id=None,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )
            assert transport.grpc_channel == mock_grpc_channel


def test_photo_path():
    place = "squid"
    photo = "clam"
    expected = "places/{place}/photos/{photo}".format(
        place=place,
        photo=photo,
    )
    actual = PlacesClient.photo_path(place, photo)
    assert expected == actual


def test_parse_photo_path():
    expected = {
        "place": "whelk",
        "photo": "octopus",
    }
    path = PlacesClient.photo_path(**expected)

    # Check that the path construction is reversible.
    actual = PlacesClient.parse_photo_path(path)
    assert expected == actual


def test_photo_media_path():
    place_id = "oyster"
    photo_reference = "nudibranch"
    expected = "places/{place_id}/photos/{photo_reference}/media".format(
        place_id=place_id,
        photo_reference=photo_reference,
    )
    actual = PlacesClient.photo_media_path(place_id, photo_reference)
    assert expected == actual


def test_parse_photo_media_path():
    expected = {
        "place_id": "cuttlefish",
        "photo_reference": "mussel",
    }
    path = PlacesClient.photo_media_path(**expected)

    # Check that the path construction is reversible.
    actual = PlacesClient.parse_photo_media_path(path)
    assert expected == actual


def test_place_path():
    place_id = "winkle"
    expected = "places/{place_id}".format(
        place_id=place_id,
    )
    actual = PlacesClient.place_path(place_id)
    assert expected == actual


def test_parse_place_path():
    expected = {
        "place_id": "nautilus",
    }
    path = PlacesClient.place_path(**expected)

    # Check that the path construction is reversible.
    actual = PlacesClient.parse_place_path(path)
    assert expected == actual


def test_review_path():
    place = "scallop"
    review = "abalone"
    expected = "places/{place}/reviews/{review}".format(
        place=place,
        review=review,
    )
    actual = PlacesClient.review_path(place, review)
    assert expected == actual


def test_parse_review_path():
    expected = {
        "place": "squid",
        "review": "clam",
    }
    path = PlacesClient.review_path(**expected)

    # Check that the path construction is reversible.
    actual = PlacesClient.parse_review_path(path)
    assert expected == actual


def test_common_billing_account_path():
    billing_account = "whelk"
    expected = "billingAccounts/{billing_account}".format(
        billing_account=billing_account,
    )
    actual = PlacesClient.common_billing_account_path(billing_account)
    assert expected == actual


def test_parse_common_billing_account_path():
    expected = {
        "billing_account": "octopus",
    }
    path = PlacesClient.common_billing_account_path(**expected)

    # Check that the path construction is reversible.
    actual = PlacesClient.parse_common_billing_account_path(path)
    assert expected == actual


def test_common_folder_path():
    folder = "oyster"
    expected = "folders/{folder}".format(
        folder=folder,
    )
    actual = PlacesClient.common_folder_path(folder)
    assert expected == actual


def test_parse_common_folder_path():
    expected = {
        "folder": "nudibranch",
    }
    path = PlacesClient.common_folder_path(**expected)

    # Check that the path construction is reversible.
    actual = PlacesClient.parse_common_folder_path(path)
    assert expected == actual


def test_common_organization_path():
    organization = "cuttlefish"
    expected = "organizations/{organization}".format(
        organization=organization,
    )
    actual = PlacesClient.common_organization_path(organization)
    assert expected == actual


def test_parse_common_organization_path():
    expected = {
        "organization": "mussel",
    }
    path = PlacesClient.common_organization_path(**expected)

    # Check that the path construction is reversible.
    actual = PlacesClient.parse_common_organization_path(path)
    assert expected == actual


def test_common_project_path():
    project = "winkle"
    expected = "projects/{project}".format(
        project=project,
    )
    actual = PlacesClient.common_project_path(project)
    assert expected == actual


def test_parse_common_project_path():
    expected = {
        "project": "nautilus",
    }
    path = PlacesClient.common_project_path(**expected)

    # Check that the path construction is reversible.
    actual = PlacesClient.parse_common_project_path(path)
    assert expected == actual


def test_common_location_path():
    project = "scallop"
    location = "abalone"
    expected = "projects/{project}/locations/{location}".format(
        project=project,
        location=location,
    )
    actual = PlacesClient.common_location_path(project, location)
    assert expected == actual


def test_parse_common_location_path():
    expected = {
        "project": "squid",
        "location": "clam",
    }
    path = PlacesClient.common_location_path(**expected)

    # Check that the path construction is reversible.
    actual = PlacesClient.parse_common_location_path(path)
    assert expected == actual


def test_client_with_default_client_info():
    client_info = gapic_v1.client_info.ClientInfo()

    with mock.patch.object(
        transports.PlacesTransport, "_prep_wrapped_messages"
    ) as prep:
        client = PlacesClient(
            credentials=ga_credentials.AnonymousCredentials(),
            client_info=client_info,
        )
        prep.assert_called_once_with(client_info)

    with mock.patch.object(
        transports.PlacesTransport, "_prep_wrapped_messages"
    ) as prep:
        transport_class = PlacesClient.get_transport_class()
        transport = transport_class(
            credentials=ga_credentials.AnonymousCredentials(),
            client_info=client_info,
        )
        prep.assert_called_once_with(client_info)


@pytest.mark.asyncio
async def test_transport_close_async():
    client = PlacesAsyncClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc_asyncio",
    )
    with mock.patch.object(
        type(getattr(client.transport, "grpc_channel")), "close"
    ) as close:
        async with client:
            close.assert_not_called()
        close.assert_called_once()


def test_transport_close():
    transports = {
        "rest": "_session",
        "grpc": "_grpc_channel",
    }

    for transport, close_name in transports.items():
        client = PlacesClient(
            credentials=ga_credentials.AnonymousCredentials(), transport=transport
        )
        with mock.patch.object(
            type(getattr(client.transport, close_name)), "close"
        ) as close:
            with client:
                close.assert_not_called()
            close.assert_called_once()


def test_client_ctx():
    transports = [
        "rest",
        "grpc",
    ]
    for transport in transports:
        client = PlacesClient(
            credentials=ga_credentials.AnonymousCredentials(), transport=transport
        )
        # Test client calls underlying transport.
        with mock.patch.object(type(client.transport), "close") as close:
            close.assert_not_called()
            with client:
                pass
            close.assert_called()


@pytest.mark.parametrize(
    "client_class,transport_class",
    [
        (PlacesClient, transports.PlacesGrpcTransport),
        (PlacesAsyncClient, transports.PlacesGrpcAsyncIOTransport),
    ],
)
def test_api_key_credentials(client_class, transport_class):
    with mock.patch.object(
        google.auth._default, "get_api_key_credentials", create=True
    ) as get_api_key_credentials:
        mock_cred = mock.Mock()
        get_api_key_credentials.return_value = mock_cred
        options = client_options.ClientOptions()
        options.api_key = "api_key"
        with mock.patch.object(transport_class, "__init__") as patched:
            patched.return_value = None
            client = client_class(client_options=options)
            patched.assert_called_once_with(
                credentials=mock_cred,
                credentials_file=None,
                host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                    UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
                ),
                scopes=None,
                client_cert_source_for_mtls=None,
                quota_project_id=None,
                client_info=transports.base.DEFAULT_CLIENT_INFO,
                always_use_jwt_access=True,
                api_audience=None,
            )
